@@ -1,5 +1,5 @@
 /* Common things used by the various *gnu-nat.c files
-   Copyright (C) 1995-2022 Free Software Foundation, Inc.
+   Copyright (C) 1995-2023 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -109,8 +109,8 @@ extern bool gnu_debug_flag;
 
 #define debug(msg, args...) \
  do { if (gnu_debug_flag) \
-	fprintf_unfiltered (gdb_stdlog, "%s:%d: " msg "\r\n", \
-			    __FILE__ , __LINE__ , ##args); } while (0)
+     gdb_printf (gdb_stdlog, "%s:%d: " msg "\r\n",		\
+		 __FILE__ , __LINE__ , ##args); } while (0)
 
 /* A prototype generic GNU/Hurd target.  The client can override it
    with local methods.  */
