@@ -1,7 +1,6 @@
 /* Target-dependent code for the i386.
 
-   Copyright (C) 2001-2022 Free Software Foundation, Inc.
-   Copyright (C) 2013-2018 Intel Corporation
+   Copyright (C) 2001-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -24,11 +23,11 @@
 #include "gdbarch.h"
 #include "infrun.h"
 #include "expression.h"
-#include "memrange.h"
+#include "i387-tdep.h"
 #include <vector>
 #include <utility>
 
-struct frame_info;
+class frame_info_ptr;
 struct gdbarch;
 struct reggroup;
 struct regset;
@@ -61,7 +60,7 @@ enum struct_return
 };
 
 /* i386 architecture specific information.  */
-struct i386_gdbarch_tdep : gdbarch_tdep
+struct i386_gdbarch_tdep : gdbarch_tdep_base
 {
   /* General-purpose registers.  */
   int *gregset_reg_offset = 0;
@@ -232,10 +231,6 @@ struct i386_gdbarch_tdep : gdbarch_tdep
      absence of segment base registers.  */
   int fsbase_regnum = 0;
 
-  /* CET registers.  */
-  int ssp_regnum;
-  int cet_msr_regnum;
-
   /* Target description.  */
   const struct target_desc *tdesc = nullptr;
 
@@ -253,10 +248,10 @@ struct i386_gdbarch_tdep : gdbarch_tdep
   CORE_ADDR sigtramp_end = 0;
 
   /* Detect sigtramp.  */
-  int (*sigtramp_p) (struct frame_info *) = nullptr;
+  int (*sigtramp_p) (frame_info_ptr) = nullptr;
 
   /* Get address of sigcontext for sigtramp.  */
-  CORE_ADDR (*sigcontext_addr) (struct frame_info *) = nullptr;
+  CORE_ADDR (*sigcontext_addr) (frame_info_ptr) = nullptr;
 
   /* Offset of registers in `struct sigcontext'.  */
   int *sc_reg_offset = 0;
@@ -271,10 +266,14 @@ struct i386_gdbarch_tdep : gdbarch_tdep
   struct type *i386_mmx_type = nullptr;
   struct type *i386_ymm_type = nullptr;
   struct type *i386_zmm_type = nullptr;
-  struct type *i386_tmm_type = nullptr;
-  struct type *i386_tilecfg_type = nullptr;
   struct type *i387_ext_type = nullptr;
   struct type *i386_bnd_type = nullptr;
+  struct type *i386_tilecfg_type = nullptr;
+
+  /* tmm registers are pseudo registers that are re-computed every time.
+     We just need to know if they have been initialized already while
+     doing this.  */
+  bool i386_tmm_type_init[I387_NUM_TMM_REGS] = { false };
 
   /* Process record/replay target.  */
   /* The map for registers because the AMD64's registers order
@@ -335,8 +334,6 @@ enum i386_regnum
   I386_ZMM0H_REGNUM,		/* %zmm0h */
   I386_ZMM7H_REGNUM = I386_ZMM0H_REGNUM + 7,
   I386_PKRU_REGNUM,
-  I386_CET_U_REGNUM,
-  I386_CET_PL3_SSP_REGNUM,
   I386_AMX_TILECFG_RAW_REGNUM,
   I386_AMX_TILEDATA_REGNUM,
   I386_FSBASE_REGNUM,
@@ -433,7 +430,7 @@ extern int i386_ax_pseudo_register_collect (struct gdbarch *gdbarch,
 #define I386_MAX_INSN_LEN (16)
 
 /* Functions exported from i386-tdep.c.  */
-extern CORE_ADDR i386_pe_skip_trampoline_code (struct frame_info *frame,
+extern CORE_ADDR i386_pe_skip_trampoline_code (frame_info_ptr frame,
 					       CORE_ADDR pc, char *name);
 extern CORE_ADDR i386_skip_main_prologue (struct gdbarch *gdbarch,
 					  CORE_ADDR pc);
@@ -452,11 +449,11 @@ extern CORE_ADDR i386_thiscall_push_dummy_call (struct gdbarch *gdbarch,
 						bool thiscall);
 
 /* Return whether the THIS_FRAME corresponds to a sigtramp routine.  */
-extern int i386_sigtramp_p (struct frame_info *this_frame);
+extern int i386_sigtramp_p (frame_info_ptr this_frame);
 
 /* Return non-zero if REGNUM is a member of the specified group.  */
 extern int i386_register_reggroup_p (struct gdbarch *gdbarch, int regnum,
-				     struct reggroup *group);
+				     const struct reggroup *group);
 
 /* Supply register REGNUM from the general-purpose register set REGSET
    to register cache REGCACHE.  If REGNUM is -1, do this for all
@@ -523,56 +520,6 @@ extern int i386_stap_is_single_operand (struct gdbarch *gdbarch,
 extern expr::operation_up i386_stap_parse_special_token
      (struct gdbarch *gdbarch, struct stap_parse_info *p);
 
-/* MSR_IA32_CET bits.  */
-#define MSR_CET_SHSTK_EN		(0x1 << 0)
-#define MSR_CET_WR_SHSTK_EN		(0x1 << 1)
-#define MSR_CET_ENDBR_EN		(0x1 << 2)
-#define MSR_CET_LEG_IW_EN		(0x1 << 3)
-#define MSR_CET_NO_TRACK_EN		(0x1 << 4)
-#define MSR_CET_SUPPRESS_DIS		(0x1 << 5)
-#define MSR_CET_SUPPRESS		(0x1 << 10)
-#define MSR_CET_TRACKER			(0x1 << 11)
-#define MSR_CET_EB_LEG_BITMAP_BASE	0xfffffffffffff000ULL
-
-/* Retrieve the mapped memory regions[ADDR_LOW, ADDR_HIGH] for a given
-   address ADDR in memory space of process TID by reading the process
-   information from its pseudo-file systema.  Return true if addr is in that
-   range.  */
-extern bool
-i386_cet_get_shstk_mem_range (const CORE_ADDR addr, mem_range *range);
-
-/* Enum for the enablement state of the feature org.gnu.gdb.i386.pl3_ssp.  */
-enum shstk_status
-{
-  /* Shadow stack is enabled for the current process.  */
-  SHSTK_ENABLED = 1,
-
-  /* Shadow stack is disabled by software for the current process.  */
-  SHSTK_DISABLED_SW = 0,
-
-  /* Shadow stack is disabled by the kernel.  */
-  SHSTK_DISABLED_KERNEL = -1,
-
-  /* Shadow stack is disabled by hardware.  */
-  SHSTK_DISABLED_HW = -2,
-};
-
-/* Check if the shadow stack is enabled.  */
-extern shstk_status
-i386_cet_shstk_state ();
-
-/* Get shadow stack pointer ssp from register cache.  */
-extern void
-i386_cet_get_shstk_pointer (struct gdbarch *gdbarch, CORE_ADDR *ssp);
-
-/* Write shadow stack pointer ssp to register cache.  */
-extern void
-i386_cet_set_shstk_pointer (struct gdbarch *gdbarch, const CORE_ADDR *ssp);
-
-/* Pushes an address to the shadow-stack and increments the ssp accordingly.  */
-extern void
-i386_cet_shstk_push (struct gdbarch *gdbarch, CORE_ADDR* new_addr);
-
 /* AMX utilities.  */
 
 /* TILECFG register.
