@@ -143,14 +143,13 @@ intelgt_read_cr0 (regcache *regcache, int subreg)
       return cr0[subreg];
 
     case REG_UNKNOWN:
-      internal_error (__FILE__, __LINE__, _("unknown register 'cr0'."));
+      internal_error (_("unknown register 'cr0'."));
 
     case REG_UNAVAILABLE:
       error (_("cr0 is not available"));
     }
 
-  internal_error (__FILE__, __LINE__, _("unknown register status: %d."),
-		  cr0status);
+  internal_error (_("unknown register status: %d."), cr0status);
 }
 
 /* Write VALUE into CR0.SUBREG in REGCACHE.  */
@@ -175,14 +174,13 @@ intelgt_write_cr0 (regcache *regcache, int subreg, uint32_t value)
       return;
 
     case REG_UNKNOWN:
-      internal_error (__FILE__, __LINE__, _("unknown register 'cr0'."));
+      internal_error (_("unknown register 'cr0'."));
 
     case REG_UNAVAILABLE:
       error (_("cr0 is not available"));
     }
 
-  internal_error (__FILE__, __LINE__, _("unknown register status: %d."),
-		  cr0status);
+  internal_error (_("unknown register status: %d."), cr0status);
 }
 
 /* Return CR0.SUBREG for TP.  */
@@ -203,6 +201,26 @@ intelgt_write_cr0 (thread_info *tp, int subreg, uint32_t value)
   intelgt_write_cr0 (regcache, subreg, value);
 }
 
+static unsigned int
+intelgt_decode_tagged_address (CORE_ADDR addr)
+{
+  /* Generic pointers are tagged in order to preserve the address space to
+     which they are pointing.  Tags are encoded into bits [61:63] of an
+     address:
+
+     000/111 - global,
+     001 - private,
+     010 - local (SLM)
+
+     We currently cannot decode this tag in GDB, as the information
+     cannot be added to the (cached) type instance flags, as it changes at
+     runtime.  */
+  if ((addr >> 61) == 0x2ul)
+    return (unsigned int) ZET_DEBUG_MEMORY_SPACE_TYPE_SLM;
+
+  return (unsigned int) ZET_DEBUG_MEMORY_SPACE_TYPE_DEFAULT;
+}
+
 /* Instruction details.  */
 
 enum
@@ -234,6 +252,12 @@ public:
   bool stopped_by_sw_breakpoint () override;
   bool supports_run_command () override;
 
+  int read_memory (CORE_ADDR memaddr, unsigned char *myaddr,
+		   int len, unsigned int addr_space = 0) override;
+
+  int write_memory (CORE_ADDR memaddr, const unsigned char *myaddr,
+		    int len, unsigned int addr_space = 0) override;
+
   CORE_ADDR read_pc (regcache *regcache) override;
   void write_pc (regcache *regcache, CORE_ADDR pc) override;
 
@@ -250,8 +274,7 @@ protected:
 
   target_stop_reason get_stop_reason (thread_info *, gdb_signal &) override;
 
-  void prepare_thread_resume (thread_info *tp,
-			      enum resume_kind rkind) override;
+  void prepare_thread_resume (thread_info *tp) override;
 
   /* Read one instruction from memory at PC into BUFFER and return the
      number of bytes read on success or a negative errno error code.
@@ -264,6 +287,16 @@ protected:
 
   bool erratum_18020355813 (thread_info *tp);
 
+  /* Read the memory in the context of thread TP.  */
+  int read_memory (thread_info *tp, CORE_ADDR memaddr,
+		   unsigned char *myaddr, int len,
+		   unsigned int addr_space = 0);
+
+  /* Write the memory in the context of thread TP.  */
+  int write_memory (thread_info *tp, CORE_ADDR memaddr,
+		    const unsigned char *myaddr, int len,
+		    unsigned int addr_space = 0);
+
 private:
   /* Add a register set for REGPROP on DEVICE to REGSETS and increment REGNUM
      accordingly.
@@ -562,6 +595,43 @@ intelgt_ze_target::get_stop_reason (thread_info *tp, gdb_signal &signal)
   return TARGET_STOPPED_BY_NO_REASON;
 }
 
+int
+intelgt_ze_target::read_memory (thread_info *tp, CORE_ADDR memaddr,
+				unsigned char *myaddr, int len,
+				unsigned int addr_space)
+{
+  if (addr_space == (unsigned int) ZET_DEBUG_MEMORY_SPACE_TYPE_DEFAULT)
+    addr_space = intelgt_decode_tagged_address (memaddr);
+
+  return ze_target::read_memory (tp, memaddr, myaddr, len, addr_space);
+}
+
+int
+intelgt_ze_target::read_memory (CORE_ADDR memaddr, unsigned char *myaddr,
+				int len, unsigned int addr_space)
+{
+  return read_memory (current_thread, memaddr, myaddr, len, addr_space);
+}
+
+int
+intelgt_ze_target::write_memory (thread_info *tp, CORE_ADDR memaddr,
+				 const unsigned char *myaddr, int len,
+				 unsigned int addr_space)
+{
+  if (addr_space == (unsigned int) ZET_DEBUG_MEMORY_SPACE_TYPE_DEFAULT)
+    addr_space = intelgt_decode_tagged_address (memaddr);
+
+  return ze_target::write_memory (tp, memaddr, myaddr, len, addr_space);
+}
+
+int
+intelgt_ze_target::write_memory (CORE_ADDR memaddr,
+				 const unsigned char *myaddr, int len,
+				 unsigned int addr_space)
+{
+  return write_memory (current_thread, memaddr, myaddr, len, addr_space);
+}
+
 int
 intelgt_ze_target::read_inst (thread_info *tp, CORE_ADDR pc,
 			      unsigned char *buffer)
@@ -706,8 +776,7 @@ intelgt_ze_target::erratum_18020355813 (thread_info *tp)
 }
 
 void
-intelgt_ze_target::prepare_thread_resume (thread_info *tp,
-					  enum resume_kind rkind)
+intelgt_ze_target::prepare_thread_resume (thread_info *tp)
 {
   ze_thread_info *zetp = ze_thread (tp);
   gdb_assert (zetp != nullptr);
@@ -730,9 +799,9 @@ intelgt_ze_target::prepare_thread_resume (thread_info *tp,
   cr0[1] &= ~(1 << intelgt_cr0_1_external_halt_status);
 
   /* Distinguish stepping and continuing.  */
-  switch (rkind)
+  switch (zetp->resume_state)
     {
-    case resume_step:
+    case ze_thread_resume_step:
       /* We step by indicating a breakpoint exception, which will be
 	 considered on the next instruction.
 
@@ -751,12 +820,12 @@ intelgt_ze_target::prepare_thread_resume (thread_info *tp,
       zetp->waitstatus.set_unavailable ();
 
       /* Fall through.  */
-    case resume_continue:
+    case ze_thread_resume_run:
       cr0[1] &= ~(1 << intelgt_cr0_1_breakpoint_status);
       break;
 
     default:
-      internal_error (__FILE__, __LINE__, _("bad resume kind: %d."), rkind);
+      internal_error (_("bad resume kind: %d."), zetp->resume_state);
     }
 
   /* When stepping over a breakpoint, we need to suppress the breakpoint
@@ -822,6 +891,8 @@ intelgt_ze_target::add_regset (target_desc *tdesc,
       intelgt_add_regset (feature, regnum, "r", regprop.count, "GRF",
 			  regprop.bitSize,
 			  intelgt_uint_reg_type (regprop.bitSize));
+
+      expedite.push_back ("r0");
       break;
 
     case ZET_DEBUG_REGSET_TYPE_ADDR_INTEL_GPU:
