@@ -1,5 +1,5 @@
 /* Internal interfaces for the GNU/Linux specific target code for gdbserver.
-   Copyright (C) 2002-2022 Free Software Foundation, Inc.
+   Copyright (C) 2002-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -25,7 +25,6 @@
 
 #include "gdbthread.h"
 #include "gdb_proc_service.h"
-#include "nonstop-low.h"
 
 /* Included for ptrace type definitions.  */
 #include "nat/linux-ptrace.h"
@@ -117,7 +116,7 @@ struct regs_info
 #endif
 };
 
-struct process_info_private : public nonstop_process_info
+struct process_info_private
 {
   /* Arch-specific additions.  */
   struct arch_process_info *arch_private;
@@ -128,13 +127,16 @@ struct process_info_private : public nonstop_process_info
 
   /* &_r_debug.  0 if not yet determined.  -1 if no PT_DYNAMIC in Phdrs.  */
   CORE_ADDR r_debug;
+
+  /* The /proc/pid/mem file used for reading/writing memory.  */
+  int mem_fd;
 };
 
 struct lwp_info;
 
 /* Target ops definitions for a Linux target.  */
 
-class linux_process_target : public nonstop_process_target
+class linux_process_target : public process_stratum_target
 {
 public:
 
@@ -155,14 +157,15 @@ public:
 
   bool thread_alive (ptid_t pid) override;
 
+  void resume (thread_resume *resume_info, size_t n) override;
+
+  ptid_t wait (ptid_t ptid, target_waitstatus *status,
+	       target_wait_flags options) override;
+
   void fetch_registers (regcache *regcache, int regno) override;
 
   void store_registers (regcache *regcache, int regno) override;
 
-  int prepare_to_access_memory () override;
-
-  void done_accessing_memory () override;
-
   int read_memory (CORE_ADDR memaddr, unsigned char *myaddr,
 		   int len, unsigned int addr_space = 0) override;
 
@@ -219,6 +222,12 @@ public:
 		     unsigned const char *writebuf,
 		     CORE_ADDR offset, int len) override;
 
+  bool supports_non_stop () override;
+
+  bool async (bool enable) override;
+
+  int start_non_stop (bool enable) override;
+
   bool supports_multi_process () override;
 
   bool supports_fork_events () override;
@@ -312,11 +321,6 @@ public:
      visibility because proc-service uses it.  */
   virtual const regs_info *get_regs_info () = 0;
 
-  virtual void low_send_sigstop (nonstop_thread_info *nti) override;
-
-  virtual void enqueue_signal_pre_resume (nonstop_thread_info *nti,
-					  int signal) override;
-
 private:
 
   /* Handle a GNU/Linux extended wait response.  If we see a clone,
@@ -354,8 +358,9 @@ private:
   /* Wait for all children to stop for the SIGSTOPs we just queued.  */
   void wait_for_sigstop ();
 
-  ptid_t low_wait (ptid_t ptid, target_waitstatus *ourstatus,
-		   target_wait_flags target_options) override;
+  /* Wait for process, returns status.  */
+  ptid_t wait_1 (ptid_t ptid, target_waitstatus *ourstatus,
+		 target_wait_flags target_options);
 
   /* Stop all lwps that aren't stopped yet, except EXCEPT, if not NULL.
      If SUSPEND, then also increase the suspend count of every LWP,
@@ -367,7 +372,16 @@ private:
      NULL.  This undoes a stop_all_lwps call.  */
   void unstop_all_lwps (int unsuspend, lwp_info *except);
 
-  void start_step_over (thread_info *thread) override;
+  /* Start a step-over operation on LWP.  When LWP stopped at a
+     breakpoint, to make progress, we need to remove the breakpoint out
+     of the way.  If we let other threads run while we do that, they may
+     pass by the breakpoint location and miss hitting it.  To avoid
+     that, a step-over momentarily stops all threads while LWP is
+     single-stepped by either hardware or software while the breakpoint
+     is temporarily uninserted from the inferior.  When the single-step
+     finishes, we reinsert the breakpoint, and let all threads that are
+     supposed to be running, run again.  */
+  void start_step_over (lwp_info *lwp);
 
   /* If there's a step over in progress, wait until all threads stop
      (that is, until the stepping thread finishes its step), and
@@ -389,7 +403,7 @@ private:
 
   /* The reason we resume in the caller, is because we want to be able
      to pass lwp->status_pending as WSTAT, and we need to clear
-     status_pending_p before resuming, otherwise, resume_one_nti
+     status_pending_p before resuming, otherwise, resume_one_lwp
      refuses to resume.  */
   bool maybe_move_out_of_jump_pad (lwp_info *lwp, int *wstat);
 
@@ -436,27 +450,39 @@ private:
      architecture.  Returns true if we now have the LWP's stop PC.  */
   bool save_stop_reason (lwp_info *lwp);
 
-  /* Resume execution of NTI.  If STEP, single-step it.  If
+  /* Resume execution of LWP.  If STEP is nonzero, single-step it.  If
      SIGNAL is nonzero, give it that signal.  */
-  void resume_one_nti_throw (nonstop_thread_info *nti, bool step,
-			     int signal, siginfo_t *info);
+  void resume_one_lwp_throw (lwp_info *lwp, int step, int signal,
+			     siginfo_t *info);
 
-  void resume_one_nti (nonstop_thread_info *nti, bool step, int signal,
-		       void *siginfo) override;
+  /* Like resume_one_lwp_throw, but no error is thrown if the LWP
+     disappears while we try to resume it.  */
+  void resume_one_lwp (lwp_info *lwp, int step, int signal, siginfo_t *info);
 
-  /* Override the base implementation to also check fork children.  */
-  bool resume_request_applies_to_thread (thread_info *thread,
-					 thread_resume &resume) override;
+  /* This function is called once per thread.  We check the thread's
+     last resume request, which will tell us whether to resume, step, or
+     leave the thread stopped.  Any signal the client requested to be
+     delivered has already been enqueued at this point.
 
-  void post_set_resume_request (thread_info *thread) override;
+     If any thread that GDB wants running is stopped at an internal
+     breakpoint that needs stepping over, we start a step-over operation
+     on that particular thread, and leave all others stopped.  */
+  void proceed_one_lwp (thread_info *thread, lwp_info *except);
 
-  void resume_stop_one_stopped_thread (nonstop_thread_info *nti) override;
+  /* This function is called once per thread.  We check the thread's
+     resume request, which will tell us whether to resume, step, or
+     leave the thread stopped; and what signal, if any, it should be
+     sent.
 
-  bool has_pending_status (nonstop_thread_info *nti) override;
+     For threads which we aren't explicitly told otherwise, we preserve
+     the stepping flag; this is used for stepping over gdbserver-placed
+     breakpoints.
 
-  void proceed_one_nti_for_resume_stop (nonstop_thread_info *nti) override;
-
-  bool resume_one_nti_should_step (nonstop_thread_info *nti) override;
+     If pending_flags was set in any thread, we queue any needed
+     signals, since we won't actually resume.  We already have a pending
+     event to report, so we don't need to preserve any step requests;
+     they should be re-issued if necessary.  */
+  void resume_one_thread (thread_info *thread, bool leave_all_stopped);
 
   /* Return true if this lwp has an interesting status pending.  */
   bool status_pending_p_callback (thread_info *thread, ptid_t ptid);
@@ -468,15 +494,29 @@ private:
   /* Unsuspend THREAD, except EXCEPT, and proceed.  */
   void unsuspend_and_proceed_one_lwp (thread_info *thread, lwp_info *except);
 
-  bool thread_still_has_status_pending (thread_info *thread) override;
+  /* Return true if this lwp still has an interesting status pending.
+     If not (e.g., it had stopped for a breakpoint that is gone), return
+     false.  */
+  bool thread_still_has_status_pending (thread_info *thread);
 
-  bool thread_needs_step_over (thread_info *thread) override;
+  /* Return true if this lwp is to-be-resumed and has an interesting
+     status pending.  */
+  bool resume_status_pending (thread_info *thread);
+
+  /* Return true if this lwp that GDB wants running is stopped at an
+     internal breakpoint that we need to step over.  It assumes that
+     any required STOP_PC adjustment has already been propagated to
+     the inferior's regcache.  */
+  bool thread_needs_step_over (thread_info *thread);
 
   /* Single step via hardware or software single step.
      Return 1 if hardware single stepping, 0 if software single stepping
      or can't single step.  */
   int single_step (lwp_info* lwp);
 
+  /* Return true if THREAD is doing hardware single step.  */
+  bool maybe_hw_step (thread_info *thread);
+
   /* Install breakpoints for software single stepping.  */
   void install_software_single_step_breakpoints (lwp_info *lwp);
 
@@ -505,6 +545,13 @@ private:
      data.  */
   process_info *add_linux_process (int pid, int attached);
 
+  /* Same as add_linux_process, but don't open the /proc/PID/mem file
+     yet.  */
+  process_info *add_linux_process_no_mem_file (int pid, int attached);
+
+  /* Free resources associated to PROC and remove it.  */
+  void remove_linux_process (process_info *proc); 
+
   /* Add a new thread.  */
   lwp_info *add_lwp (ptid_t ptid);
 
@@ -578,8 +625,6 @@ protected:
 
   virtual void low_set_pc (regcache *regcache, CORE_ADDR newpc);
 
-  virtual bool supports_breakpoints () override final;
-
   /* Find the next possible PCs after the current instruction executes.
      Targets that override this method should also override
      'supports_software_single_step' to return true.  */
@@ -685,7 +730,7 @@ struct pending_signal
    There is also ``all_processes'' is keyed by the "overall process ID",
    which GNU/Linux calls tgid, "thread group ID".  */
 
-struct lwp_info : public nonstop_thread_info
+struct lwp_info
 {
   /* If this LWP is a fork child that wasn't reported to GDB yet, return
      its parent, else nullptr.  */
@@ -740,10 +785,25 @@ struct lwp_info : public nonstop_thread_info
     return this->fork_relative;
   }
 
+  /* Backlink to the parent object.  */
+  struct thread_info *thread = nullptr;
+
+  /* If this flag is set, the next SIGSTOP will be ignored (the
+     process will be immediately resumed).  This means that either we
+     sent the SIGSTOP to it ourselves and got some other pending event
+     (so the SIGSTOP is still pending), or that we stopped the
+     inferior implicitly via PTRACE_ATTACH and have not waited for it
+     yet.  */
+  int stop_expected = 0;
+
   /* When this is true, we shall not try to resume this thread, even
      if last_resume_kind isn't resume_stop.  */
   int suspended = 0;
 
+  /* If this flag is set, the lwp is known to be stopped right now (stop
+     event already received in a wait()).  */
+  int stopped = 0;
+
   /* Signal whether we are in a SYSCALL_ENTRY or
      in a SYSCALL_RETURN event.
      Values:
@@ -776,6 +836,10 @@ struct lwp_info : public nonstop_thread_info
   int status_pending_p = 0;
   int status_pending = 0;
 
+  /* The reason the LWP last stopped, if we need to track it
+     (breakpoint, watchpoint, etc.)  */
+  enum target_stop_reason stop_reason = TARGET_STOPPED_BY_NO_REASON;
+
   /* On architectures where it is possible to know the data address of
      a triggered watchpoint, STOPPED_DATA_ADDRESS is non-zero, and
      contains such data address.  Only valid if STOPPED_BY_WATCHPOINT
@@ -790,6 +854,12 @@ struct lwp_info : public nonstop_thread_info
      level on this process was a single-step.  */
   int stepping = 0;
 
+  /* Range to single step within.  This is a copy of the step range
+     passed along the last resume request.  See 'struct
+     thread_resume'.  */
+  CORE_ADDR step_range_start = 0; /* Inclusive */
+  CORE_ADDR step_range_end = 0; /* Exclusive */
+
   /* If this flag is set, we need to set the event request flags the
      next time we see this LWP stop.  */
   int must_set_ptrace_flags = 0;
@@ -797,6 +867,10 @@ struct lwp_info : public nonstop_thread_info
   /* A chain of signals that need to be delivered to this process.  */
   std::list<pending_signal> pending_signals;
 
+  /* A link used when resuming.  It is initialized from the resume request,
+     and then processed and cleared in linux_resume_one_lwp.  */
+  struct thread_resume *resume = nullptr;
+
   /* Information bout this lwp's fast tracepoint collection status (is it
      currently stopped in the jump pad, and if so, before or at/after the
      relocated instruction).  Normally, we won't care about this, but we will
