@@ -60,9 +60,9 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/jk/workspace/acl/pipeline/dep/include',
  'CONFINCLUDEPY': '/jk/workspace/acl/pipeline/dep/include/python3.11',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/jk/workspace/acl/pipeline/build/python/Python-3.11.2/coverage.info',
+ 'COVERAGE_INFO': '/jk/workspace/acl/pipeline/build/python/Python-3.11.4/coverage.info',
  'COVERAGE_LCOV_OPTIONS': '--rc lcov_branch_coverage=1',
- 'COVERAGE_REPORT': '/jk/workspace/acl/pipeline/build/python/Python-3.11.2/lcov-report',
+ 'COVERAGE_REPORT': '/jk/workspace/acl/pipeline/build/python/Python-3.11.4/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--rc lcov_branch_coverage=1 --branch-coverage '
                             '--title "CPython 3.11 LCOV report [commit $(shell '
                             ')]"',
@@ -161,7 +161,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_CONFSTR': 1,
  'HAVE_CONIO_H': 0,
  'HAVE_CONNECT': 1,
- 'HAVE_COPY_FILE_RANGE': 0,
+ 'HAVE_COPY_FILE_RANGE': 1,
  'HAVE_CRYPT_H': 1,
  'HAVE_CRYPT_R': 1,
  'HAVE_CTERMID': 1,
@@ -210,7 +210,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_ERRNO_H': 1,
  'HAVE_EVENTFD': 1,
  'HAVE_EXECV': 1,
- 'HAVE_EXPLICIT_BZERO': 0,
+ 'HAVE_EXPLICIT_BZERO': 1,
  'HAVE_EXPLICIT_MEMSET': 0,
  'HAVE_EXPM1': 1,
  'HAVE_FACCESSAT': 1,
@@ -252,7 +252,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETC_UNLOCKED': 1,
  'HAVE_GETEGID': 1,
- 'HAVE_GETENTROPY': 0,
+ 'HAVE_GETENTROPY': 1,
  'HAVE_GETEUID': 1,
  'HAVE_GETGID': 1,
  'HAVE_GETGRGID': 1,
@@ -283,7 +283,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_GETPWNAM_R': 1,
  'HAVE_GETPWUID': 1,
  'HAVE_GETPWUID_R': 1,
- 'HAVE_GETRANDOM': 0,
+ 'HAVE_GETRANDOM': 1,
  'HAVE_GETRANDOM_SYSCALL': 1,
  'HAVE_GETRESGID': 1,
  'HAVE_GETRESUID': 1,
@@ -341,7 +341,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_LINUX_CAN_RAW_JOIN_FILTERS': 1,
  'HAVE_LINUX_MEMFD_H': 1,
  'HAVE_LINUX_NETLINK_H': 1,
- 'HAVE_LINUX_QRTR_H': 0,
+ 'HAVE_LINUX_QRTR_H': 1,
  'HAVE_LINUX_RANDOM_H': 1,
  'HAVE_LINUX_SOUNDCARD_H': 1,
  'HAVE_LINUX_TIPC_H': 1,
@@ -359,8 +359,8 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_MADVISE': 1,
  'HAVE_MAKEDEV': 1,
  'HAVE_MBRTOWC': 1,
- 'HAVE_MEMFD_CREATE': 0,
- 'HAVE_MEMORY_H': 1,
+ 'HAVE_MEMFD_CREATE': 1,
+ 'HAVE_MEMORY_H': 0,
  'HAVE_MEMRCHR': 1,
  'HAVE_MKDIRAT': 1,
  'HAVE_MKFIFO': 1,
@@ -397,7 +397,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_POSIX_SPAWNP': 1,
  'HAVE_PREAD': 1,
  'HAVE_PREADV': 1,
- 'HAVE_PREADV2': 0,
+ 'HAVE_PREADV2': 1,
  'HAVE_PRLIMIT': 1,
  'HAVE_PROCESS_H': 0,
  'HAVE_PROTOTYPES': 1,
@@ -412,7 +412,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_PTY_H': 1,
  'HAVE_PWRITE': 1,
  'HAVE_PWRITEV': 1,
- 'HAVE_PWRITEV2': 0,
+ 'HAVE_PWRITEV2': 1,
  'HAVE_READLINK': 1,
  'HAVE_READLINKAT': 1,
  'HAVE_READV': 1,
@@ -535,7 +535,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_SYS_NDIR_H': 0,
  'HAVE_SYS_PARAM_H': 1,
  'HAVE_SYS_POLL_H': 1,
- 'HAVE_SYS_RANDOM_H': 0,
+ 'HAVE_SYS_RANDOM_H': 1,
  'HAVE_SYS_RESOURCE_H': 1,
  'HAVE_SYS_SELECT_H': 1,
  'HAVE_SYS_SENDFILE_H': 1,
@@ -689,7 +689,7 @@ build_time_vars = {'ABIFLAGS': '',
  'MACOSX_DEPLOYMENT_TARGET': '',
  'MAINCC': 'gcc -pthread',
  'MAJOR_IN_MKDEV': 0,
- 'MAJOR_IN_SYSMACROS': 0,
+ 'MAJOR_IN_SYSMACROS': 1,
  'MAKESETUP': './Modules/makesetup',
  'MANDIR': '/jk/workspace/acl/pipeline/dep/share/man',
  'MKDIR_P': '/bin/mkdir -p',
@@ -813,8 +813,10 @@ build_time_vars = {'ABIFLAGS': '',
  'MODULE__ELEMENTTREE_STATE': 'yes',
  'MODULE__FUNCTOOLS_LDFLAGS': '',
  'MODULE__GDBM_STATE': 'missing',
+ 'MODULE__HASHLIB_CFLAGS': '',
  'MODULE__HASHLIB_DEPS': './Modules/hashlib.h',
- 'MODULE__HASHLIB_STATE': 'missing',
+ 'MODULE__HASHLIB_LDFLAGS': 'o',
+ 'MODULE__HASHLIB_STATE': 'yes',
  'MODULE__HEAPQ_STATE': 'yes',
  'MODULE__IO_CFLAGS': '-I./Modules/_io',
  'MODULE__IO_DEPS': './Modules/_io/_iomodule.h',
@@ -860,11 +862,13 @@ build_time_vars = {'ABIFLAGS': '',
                          './Modules/_sqlite/row.h ./Modules/_sqlite/util.h',
  'MODULE__SQLITE3_STATE': 'disabled',
  'MODULE__SRE_LDFLAGS': '',
+ 'MODULE__SSL_CFLAGS': '',
  'MODULE__SSL_DEPS': './Modules/_ssl.h ./Modules/_ssl/cert.c '
                      './Modules/_ssl/debughelpers.c ./Modules/_ssl/misc.c '
                      './Modules/_ssl_data.h ./Modules/_ssl_data_111.h '
                      './Modules/_ssl_data_300.h ./Modules/socketmodule.h',
- 'MODULE__SSL_STATE': 'missing',
+ 'MODULE__SSL_LDFLAGS': '-lssl o',
+ 'MODULE__SSL_STATE': 'yes',
  'MODULE__STATISTICS_LDFLAGS': '-lm',
  'MODULE__STATISTICS_STATE': 'yes',
  'MODULE__STAT_LDFLAGS': '',
@@ -1078,8 +1082,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/jk/workspace/acl/pipeline/build/python/Python-3.11.2',
- 'abs_srcdir': '/jk/workspace/acl/pipeline/build/python/Python-3.11.2',
+ 'abs_builddir': '/jk/workspace/acl/pipeline/build/python/Python-3.11.4',
+ 'abs_srcdir': '/jk/workspace/acl/pipeline/build/python/Python-3.11.4',
  'datarootdir': '/jk/workspace/acl/pipeline/dep/share',
  'exec_prefix': '/jk/workspace/acl/pipeline/dep',
  'prefix': '/jk/workspace/acl/pipeline/dep',
