@@ -1,5 +1,4 @@
-# Copyright (C) 2019-2022 Free Software Foundation, Inc.
-# Copyright (C) 2020-2022 Intel Corporation
+# Copyright (C) 2019-2023 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,59 +16,35 @@
 """Auto-attach to debug kernels offloaded to Intel GPU
 
 The python script allows the debugger to spawn an instance of
-'gdbserver-gt' or 'gdbserver-ze' to listen to debug events from the
-GPU.  This is done to ensure debuggability in case the kernel is
-offloaded to GPU.  The auto-attach feature is turned on by default,
-and it is harmless to the debugging capability on the CPU device.
+'gdbserver-ze' to listen to debug events from the GPU.  This is done
+to ensure debuggability in case the kernel is offloaded to GPU.  The
+auto-attach feature is turned on by default, and it is harmless to the
+debugging capability on the CPU device.
 
 The auto-attach feature can be disabled by setting the OS environment
 variable 'INTELGT_AUTO_ATTACH_DISABLE' to a non-zero value.  It can be
 also set or unset in the inferior's environment, so that the user can
 disable or enable auto-attach in the same gdb session.
-'DISABLE_AUTO_ATTACH' is marked as deprecated and ignored.
 
-By default, it is expected that the 'gdbserver-gt' executable exists
+By default, it is expected that the 'gdbserver-ze' executable exists
 in the PATH.  Below are customization options via environment
 variables.
 
- * To use a 'gdbserver-gt' that resides in a specific directory <DIR>,
-   set the INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH environment variable
+ * To use a 'gdbserver-ze' that resides in a specific directory <DIR>,
+   set the INTELGT_AUTO_ATTACH_GDBSERVER_PATH environment variable
    to <DIR>.
+   'INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH' is marked as deprecated.
 
- * The content of the INTELGT_AUTO_ATTACH_IGFXDBG_LIB_PATH environment
-   variable is prepended to the LD_LIBRARY_PATH environment variable
-   when spawning 'gdbserver-gt'.
-
- * For remote-debugging scenarios, 'gdbserver-gt' is spawned in the
-   remote machine, via the 'ssh -T' protocol and using the current
-   username.  The remote machine address is extracted from GDB.
-
-   - To change the protocol, set the INTELGT_AUTO_ATTACH_REMOTE_PROTOCOL
-     environment variable to the desired value.
-
-   - To change the username, set the INTELGT_AUTO_ATTACH_REMOTE_USERNAME
-     environment variable to the desired value.
-
-   - To change the remote machine address, set the
-     INTELGT_AUTO_ATTACH_REMOTE_TARGET environment variable to the
-     desired value.
-
-   - To enable developer debug logs of the python script set the
-     INTELGT_AUTO_ATTACH_VERBOSE_LOG environment variable to the
-     desired value.
+ * To enable developer debug logs of the python script set the
+   INTELGT_AUTO_ATTACH_VERBOSE_LOG environment variable to the
+   desired value.
 """
 
 import re
 import subprocess
+import platform
 import gdb
 
-# Breakpoint function for executing the auto-attach.
-BP_FCT = "igfxdbgxchgDebuggerHook"
-BP_ZET = "zeContextCreate"
-
-# Function call to query the number of devices.
-NUM_DEVICES_FCT = "(int)igfxdbgxchgNumDevices()"
-
 class DebugLogger:
     """Class for convenient debug logs"""
     @staticmethod
@@ -93,7 +68,7 @@ class DebugLogger:
                 for arg in args:
                     match = re.search(
                         f"<(?:{str(func.__module__)}.)?(.+) "
-                         "object at 0x[0-9a-fA-F]*>", repr(arg))
+                        "object at 0x[0-9a-fA-F]*>", repr(arg))
                     if match and self_name in match.group(1):
                         arg_repr.append("self")
                     else:
@@ -117,8 +92,110 @@ class DebugLogger:
         if DebugLogger.debug_log_is_enabled():
             print(f"intelgt: {msg}")
 
+class IntelgtErrorReport():
+    """Class to handle and highlight intelgt GPU debugging errors."""
+
+    @staticmethod
+    def exit_intelgt_session(
+            input_err=None, exception=None, terminate=True):
+        """Highlight the error and give the option to user for ending the GPU
+        Debugging session."""
+        if exception:
+            # Color the exception message in YELLOW color for the better
+            # visibility of issue.
+            print(f"\033[93m{str(exception)}\033[93m")
+
+        # Print remaining error messages in RED color.
+        print("\033[31m")
+
+        if input_err:
+            print(f"{input_err}")
+
+        print("""
+ERROR: gdbserver-ze failed to start.  Intel GPU debugging is not possible.
+Set the environment variable INTELGT_AUTO_ATTACH_DISABLE=1 for disabling auto-attach.
+\033[0m""")
+
+        try:
+            if terminate:
+                gdb.execute("quit")
+
+        # In case user enters "n" for quit command, exception is triggered.
+        except RuntimeError:
+            print("""\033[31m
+Continuing with GPU-debugging disabled.
+\033[0m""")
+
+    @staticmethod
+    def execute_with_errorcapture(command, capture_output=True):
+        """Run gdb.execute with to_string option set to True to capture the
+        output from gdbserver.  In case of exception, highlight the error."""
+        try:
+            gdb.execute(f"{command}", False, capture_output)
+            return True
+        except gdb.error as gdb_exception:
+            IntelgtErrorReport.exit_intelgt_session(exception=gdb_exception)
+            return False
+
+    @staticmethod
+    def is_level_zero_debugging_enabled():
+        """Gets environment variable 'ZET_ENABLE_PROGRAM_DEBUGGING' value and
+        returns True if it is enabled otherwise returns False."""
+        env_var = IntelgtAutoAttach.get_env_variable(
+            "ZET_ENABLE_PROGRAM_DEBUGGING", "0")
+
+        return env_var != "0"
+
+    @staticmethod
+    def get_kmd_debug_enabled_status():
+        """ Return True if prelim_enable_eu_debug kernel settings is set to 1
+        for all GPU devices otherwise it returns False."""
+        try:
+            """Check i915 prelim_enable_eu_debug settings."""
+            # Settings are not yet known for Windows so return True for now.
+            if platform.system() == "Windows":
+                return True
+            if platform.system() != "Linux":
+                return False
+            find_out = subprocess.check_output(
+                "find /sys/devices | grep enable_eu_debug",
+                shell=True).decode()
+            lines = find_out.split('\n')
+            if len(lines) == 0:
+                return False
+
+            # All GPUs should have prelim_enable_eu_debug set to 1.
+            for line in lines:
+                if line == "":
+                    continue
+                debug_enabled = subprocess.run(["cat", line],
+                        capture_output=True, text=True,
+                        check=True).stdout
+                if "0" in debug_enabled:
+                    return False
+            return True
+        except (RuntimeError, subprocess.CalledProcessError):
+            return False
+
+    @staticmethod
+    def is_initialization_error():
+        """Raise the error with appropriate reason if any pre-requisite is
+        missing for the intelgt GPU debugging.  It is called before attempting
+        to attach gdbserver-ze."""
+        if not IntelgtErrorReport.is_level_zero_debugging_enabled():
+            IntelgtErrorReport.exit_intelgt_session("""
+Set 'ZET_ENABLE_PROGRAM_DEBUGGING=1' in host process environment.""")
+            return True
+        if not IntelgtErrorReport.get_kmd_debug_enabled_status():
+            IntelgtErrorReport.exit_intelgt_session("""
+Set kernel parameter 'i915.enable_eu_debug=1' or set /sys/class/drm/card*/prelim_enable_eu_debug system files value to 1.""")
+            return True
+
+        return False
+
+
 # pylint: disable-next=too-many-instance-attributes
-class IntelgtAutoAttach(gdb.Breakpoint):
+class IntelgtAutoAttach:
     """Class to debug kernels offloaded to Intel GPU"""
     def __init__(self):
         """Registration of new loaded object file handler event."""
@@ -130,12 +207,7 @@ class IntelgtAutoAttach(gdb.Breakpoint):
         self.host_inf_for_auto_remove = None
         self.is_nonstop = False
         self.hook_bp = None
-        env_value = self.get_env_variable("ZET_ENABLE_PROGRAM_DEBUGGING")
-        self.use_dcd = env_value is None or env_value == "0"
-        if self.use_dcd:
-            self.gdbserver_gt_binary = "gdbserver-gt"
-        else:
-            self.gdbserver_gt_binary = "gdbserver-ze"
+        self.the_bp = None
         self.enable_schedule_multiple_at_gt_removal = False
         # Env variable to pass custom flags to gdbserver such as
         # "--debug" and "--remote-debug" for debugging purposes.
@@ -153,69 +225,60 @@ class IntelgtAutoAttach(gdb.Breakpoint):
             return default
         return env_var.replace(f"{var} = ", "", 1).strip()
 
-    def is_auto_attach_disabled(self):
-        """Helper function to check if auto-attach has been disabled via
-        os environment variable or in inferior's environment."""
-        if self.get_env_variable("DISABLE_AUTO_ATTACH") is not None:
-            print("""\
-intelgt: env variable 'DISABLE_AUTO_ATTACH' is deprecated.  Use
-'INTELGT_AUTO_ATTACH_DISABLE' instead.""")
-        env_value = self.get_env_variable("INTELGT_AUTO_ATTACH_DISABLE")
-        return not(env_value is None or env_value == "0")
-
-    @staticmethod
-    def ze_context_is_initialized():
-        """Helper function to check if context is initialized already before
-        GDB attaches to the process."""
-
-        # If we attached to the inferior, L0 has likely been
-        # initialized already.  So, this check can cover a large range
-        # of use-cases; however, it is still incomplete, because the
-        # user could have attached to the host inferior after it was
-        # started but before L0 was initialized.  If that's the case,
-        # our attempt to attach to the GPU would fail.
-        return gdb.selected_inferior().was_attached
-
     def handle_new_objfile_event(self, event):
         """Handler for a new object file load event.  If auto-attach has
         not been disabled, set a breakpoint at location 'BP_FCT'."""
         if event is None:
             return
 
-        if not ('libigfxdbgxchg64.so' in event.new_objfile.filename or
-                'libze_loader.so' in event.new_objfile.filename or
-                'libze_intel_gpu.so' in event.new_objfile.filename):
+        # For some OSes, .gnu_debugdata section makes GDB create an in-memory
+        # debug object file.  Ignore it and wait for the actual object file.
+        # See gdb/minidebug.c.
+        if event.new_objfile.filename.startswith(".gnu_debugdata for "):
+            DebugLogger.log(
+                f"Skipping hook breakpoint for {event.new_objfile.filename} loaded event.")
             return
 
-        if self.is_auto_attach_disabled():
+        if not ('libze_loader.so' in event.new_objfile.filename or
+                'libze_intel_gpu.so' in event.new_objfile.filename or
+                'ze_loader.dll' in event.new_objfile.filename or
+                'ze_intel_gpu64.dll' in event.new_objfile.filename):
             return
 
-        if ('libze_intel_gpu.so' in event.new_objfile.filename
-            and not self.use_dcd):
+        if ('libze_intel_gpu.so' in event.new_objfile.filename or
+            'ze_intel_gpu64.dll' in event.new_objfile.filename):
             DebugLogger.log(
                 f"received {event.new_objfile.filename} loaded event.")
-            if self.ze_context_is_initialized():
-                # We just learnt about the library event, but the
-                # context is already created.  This must be because we
-                # were attached to an already-running process.
-                # Initialize the gt inferiors immediately.
+            if gdb.selected_inferior().was_attached:
+                # We just learnt about the library event and the inferior
+                # was attached.  It is possible that the level-zero backend
+                # is already initialized.  Let us try to initialize the
+                # GT inferiors.  If this does not work, it means level-zero
+                # initialization has not happened yet.  In that case we try
+                # again later via the hook BP.
                 DebugLogger.log("context might already be initialized.")
                 self.init_gt_inferiors()
             return
 
-        if ('libigfxdbgxchg64.so' in event.new_objfile.filename
-            and self.use_dcd):
-            the_bp = BP_FCT
-        elif ('libze_loader.so' in event.new_objfile.filename
-              and not self.use_dcd):
-            the_bp = BP_ZET
+        if ('libze_loader.so' in event.new_objfile.filename or
+            'ze_loader.dll' in event.new_objfile.filename):
+            self.the_bp = "zeContextCreate"
         else:
             return
 
         DebugLogger.log(
             f"received {event.new_objfile.filename} loaded event. "
-             "Setting up bp hook.")
-        self.hook_bp = gdb.Breakpoint(the_bp, type=gdb.BP_BREAKPOINT,
+            "Setting up bp hook.")
+        self.setup_hook_bp()
+
+    def setup_hook_bp(self):
+        """Set a breakpoint at the location indicated by self.the_bp.  If
+        not set, return and do nothing."""
+        if not self.the_bp:
+            DebugLogger.log("BP location for the hook is not set.")
+            return
+
+        self.hook_bp = gdb.Breakpoint(self.the_bp, type=gdb.BP_BREAKPOINT,
                                       internal=1, temporary=0)
         self.hook_bp.silent = True
         commands = ("python gdb.function.intelgt_auto_attach" +
@@ -265,19 +328,6 @@ intelgt: env variable 'DISABLE_AUTO_ATTACH' is deprecated.  Use
             if gt_inf.pid != 0:
                 gdb.execute(f"detach inferiors {gt_inf.num}", False, False)
 
-        # gdbserver-gt uses the --multi mode whereas gdbserver-ze uses
-        # --attach.  We need to explicitly make gdbserver-gt quit.
-        if self.use_dcd:
-            # Terminate gdbserver.  We should do this from a gt inf.
-            if gt_infs and gdb.selected_inferior() not in gt_infs:
-                cmd = f"inferior {gt_infs[0].num}"
-                self.protected_gdb_execute(cmd, True)
-
-            # Check if switching to gt inf was successful.
-            # Otherwise, we cannot send 'monitor exit' command.
-            if gdb.selected_inferior() in gt_infs:
-                gdb.execute("monitor exit")
-
         # Switch to host to be able to remove gt infs.
         # Catch exceptions, since the switch will likely raise
         # an error because the host inf also exited.
@@ -291,7 +341,7 @@ intelgt: env variable 'DISABLE_AUTO_ATTACH' is deprecated.  Use
             gt_inf_num = gt_inf.num
             self.protected_gdb_execute(f"remove-inferiors {gt_inf_num}")
             print(f"intelgt: inferior {gt_inf_num} "
-                  f"({self.gdbserver_gt_binary}) has been removed.")
+                  "(gdbserver-ze) has been removed.")
 
         self.set_suppress_notifications("on" if cli_suppressed else "off")
 
@@ -396,11 +446,10 @@ intelgt: env variable 'DISABLE_AUTO_ATTACH' is deprecated.  Use
             self.remove_gt_inf_if_stored_for_removal()
 
     @DebugLogger.log_call
-    def handle_error(self, inf):
+    def handle_error(self, inf, details=None):
         """Remove the inferior and clean-up dict in case of error."""
-        print(f"""\
-intelgt: {self.gdbserver_gt_binary} failed to start.  The environment variable
-INTELGT_AUTO_ATTACH_DISABLE=1 can be used for disabling auto-attach.""")
+        if details is not None:
+            print(details)
         gdb.execute(f"inferior {inf.num}", False, True)
         # The gt inferior is the last inferior that was added.
         gt_inf = gdb.inferiors()[-1]
@@ -424,59 +473,56 @@ INTELGT_AUTO_ATTACH_DISABLE=1 can be used for disabling auto-attach.""")
         return connection_str
 
     @DebugLogger.log_call
-    def handle_attach_gdbserver_gt(self, connection, inf):
+    def handle_attach_gdbserver_gt(self, inf):
         """Attach gdbserver to gt inferior either locally or remotely."""
-        # Check 'INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH' in the inferior's
-        # environment.
-        gdbserver_gt_path_env_var = \
-          self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH")
 
-        binary = self.gdbserver_gt_binary + self.gdbserver_args
-        if self.use_dcd:
-            gdbserver_gt_attach_str = f"{binary} --multi --hostpid={inf.pid} -"
+        # User may disable the auto attach from GDB command prompt.
+        if self.get_env_variable("INTELGT_AUTO_ATTACH_DISABLE", "0") == "1":
+            return False
+
+        if IntelgtErrorReport.is_initialization_error():
+            return False
+
+        # Check 'INTELGT_AUTO_ATTACH_GDBSERVER_PATH' in the inferior's
+        # environment.
+        if self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH") is not None:
+            print("""\033[31m
+INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH is deprecated. Use INTELGT_AUTO_ATTACH_GDBSERVER_PATH instead.
+\033[0m""")
+            gdbserver_path_env_var = \
+                    self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH")
+        gdbserver_path_env_var = \
+          self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_PATH")
+
+        binary = "gdbserver-ze " + self.gdbserver_args
+        if platform.system() == "Windows":
+            gdbserver_attach_str = f"{binary} --attach 127.0.0.1:PORT_PLACE_HOLDER {inf.pid}"
         else:
-            gdbserver_gt_attach_str = f"{binary} --attach - {inf.pid}"
+            gdbserver_attach_str = f"{binary} --attach - {inf.pid}"
 
-        if gdbserver_gt_path_env_var:
-            gdbserver_gt_attach_str = \
-              f"{gdbserver_gt_path_env_var}/{gdbserver_gt_attach_str}"
-
-        # Check 'INTELGT_AUTO_ATTACH_IGFXDBG_LIB_PATH' in the inferior's
-        # environment.
-        igfxdbg_lib_path_env_var = \
-          self.get_env_variable("INTELGT_AUTO_ATTACH_IGFXDBG_LIB_PATH")
-        ld_lib_path_str = ""
-        if igfxdbg_lib_path_env_var:
-            ld_lib_path_str = \
-                f"LD_LIBRARY_PATH={igfxdbg_lib_path_env_var}:$LD_LIBRARY_PATH"
+        if gdbserver_path_env_var:
+            gdbserver_attach_str = \
+              f"{gdbserver_path_env_var}/{gdbserver_attach_str}"
 
         try:
-            if connection == 'native':
-                self.make_native_gdbserver(inf, ld_lib_path_str,
-                                           gdbserver_gt_attach_str)
-            elif connection == 'remote':
-                self.make_remote_gdbserver(inf, ld_lib_path_str,
-                                           gdbserver_gt_attach_str)
+            self.make_native_gdbserver(inf, gdbserver_attach_str)
         except gdb.error as ex:
             """Explicitly raise exception to 'init_gt_inferior'.  This
             otherwise results in undhandled exception in 'init_gt_inferior'
             if ctrl-c is used whilst attaching to inferior.  """
             raise ex
 
+        return True
+
     @DebugLogger.log_call
-    def make_native_gdbserver(self, inf, ld_lib_path, gdbserver_cmd):
+    def make_native_gdbserver(self, inf, gdbserver_cmd):
         """Spawn and connect to a native instance of gdbserver."""
         # Switch to the gt inferior.  It is the most recent inferior.
         gt_inf = gdb.inferiors()[-1]
         gdb.execute(f"inferior {gt_inf.num}", False, True)
         gdb.execute("set sysroot /")
         target_remote_str = \
-            " ".join((f"{ld_lib_path} {gdbserver_cmd}").split())
-
-        if self.use_dcd:
-            target_cmd = "target extended-remote"
-        else:
-            target_cmd = "target remote"
+            " ".join((f"{gdbserver_cmd}").split())
 
         # If the user set args for gdbserver, do not capture and suppress
         # the output, because the user wants to see the full output.
@@ -484,89 +530,22 @@ INTELGT_AUTO_ATTACH_DISABLE=1 can be used for disabling auto-attach.""")
 
         gdbserver_port = \
           self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_PORT")
+        if platform.system() == "Windows":
+            if not gdbserver_port:
+                print("intelgt: Setting port to default 1234.")
+                gdbserver_port = "1234"
+            target_remote_str = target_remote_str.replace("PORT_PLACE_HOLDER", gdbserver_port)
         if gdbserver_port:
-            print(f"intelgt: connecting to {self.gdbserver_gt_binary} "
+            print(f"intelgt: connecting to gdbserver-ze "
                   f"on port {gdbserver_port}.")
-            gdb.execute(f"{target_cmd} :{gdbserver_port}", False,
-                        capture_output)
+            IntelgtErrorReport.execute_with_errorcapture(
+                f"target remote :{gdbserver_port}", capture_output)
         else:
-            gdb.execute(f"{target_cmd} | {target_remote_str}", False,
-                        capture_output)
-            print(f"intelgt: {self.gdbserver_gt_binary} started for "
+            IntelgtErrorReport.execute_with_errorcapture(
+                f"target remote | {target_remote_str}", capture_output)
+            print(f"intelgt: gdbserver-ze started for "
                   f"process {inf.pid}.")
 
-        # gdbserver-gt uses the --multi mode.  We need to make a pending attach.
-        if self.use_dcd:
-            # Set a pending attach.
-            connection_output = gdb.execute("attach 0", False, True)
-            # Print the first line only.  It contains useful device id and
-            # architecture info.
-            print(connection_output.split("\n")[0])
-
-    @DebugLogger.log_call
-    def make_remote_gdbserver(self, inf, ld_lib_path, gdbserver_cmd):
-        """Spawn and connect to a remote instance of gdbserver."""
-        # Check 'INTELGT_AUTO_ATTACH_REMOTE_TARGET' in the inferior's
-        # environment.
-        remote_target_env_var = self.get_env_variable(
-            "INTELGT_AUTO_ATTACH_REMOTE_TARGET")
-
-        # Check 'INTELGT_AUTO_ATTACH_REMOTE_USERNAME' in the inferior's
-        # environment.
-        remote_username_str = self.get_env_variable(
-            "INTELGT_AUTO_ATTACH_REMOTE_USERNAME", "")
-        if remote_username_str != "":
-            remote_username_str += "@"
-
-        # Check 'INTELGT_AUTO_ATTACH_REMOTE_PROTOCOL' in the inferior's
-        # environment.
-        remote_protocol_str = self.get_env_variable(
-            "INTELGT_AUTO_ATTACH_REMOTE_PROTOCOL", "ssh -T")
-
-        # Switch to the gt inferior.
-        gt_inf = gdb.inferiors()[-1]
-        gdb.execute(f"inferior {gt_inf.num}", False, True)
-
-        if not remote_target_env_var:
-            connection_target = inf.connection_target
-            if connection_target is None:
-                # Connection over stdio pipe.
-                target_str = self.get_connection_str(inf.connection_string)
-                if target_str is None:
-                    print("intelgt: could not get connection string.")
-                    self.handle_error(inf)
-                    return
-            else:
-                # Connection over tcp.
-                target_str = f"{remote_protocol_str} " \
-                             f"{remote_username_str}{connection_target}"
-                gdb.execute("set remotetimeout 20")
-        else:
-            # Connection over stdio pipe or tcp.
-            target_str = f"{remote_protocol_str} " \
-                         f"{remote_username_str}{remote_target_env_var}"
-            gdb.execute("set remotetimeout 20")
-
-        target_remote_str = \
-            " ".join((f"{target_str} {ld_lib_path} {gdbserver_cmd}").split())
-
-        gdb.execute(f"target extended-remote | {target_remote_str}")
-        print(f"intelgt: inferior {gt_inf.num} "
-              f"created remotely for process {inf.pid} "
-              f"using command '{target_remote_str}'")
-
-    @staticmethod
-    def igfxdcd_is_loaded():
-        """Check if the igfxdcd module is loaded."""
-        # Use lsmod to see if the igfxdcd module exists.  We don't use
-        # modinfo because in case the module was loaded as a .ko file
-        # via insmod, modinfo may not report it.
-        with subprocess.Popen(['lsmod'], stdout=subprocess.PIPE) as lsmod:
-            with subprocess.Popen(['grep', "^igfxdcd "], stdin=lsmod.stdout,
-                                  stdout=subprocess.DEVNULL) as grep:
-                grep.communicate()
-                return grep.returncode == 0
-
     @DebugLogger.log_call
     def init_gt_inferiors(self):
         """Create/initialize inferiors with gdbserver connection"""
@@ -581,24 +560,15 @@ INTELGT_AUTO_ATTACH_DISABLE=1 can be used for disabling auto-attach.""")
         host_inf = gdb.selected_inferior()
 
         connection = host_inf.connection.type
-        if connection not in ('remote', 'native'):
+        if connection != 'native':
             print(f"intelgt: connection name '{connection}' not recognized.")
             self.handle_error(host_inf)
+            IntelgtErrorReport.exit_intelgt_session(f"""
+Connection name '{connection}' not recognized.""")
             return
 
         self.hook_bp.delete()
 
-        if self.use_dcd:
-            if not self.igfxdcd_is_loaded():
-                print("""\
-intelgt: the igfxdcd module (i.e. the debug driver) is not loaded.""")
-                return
-
-            num_devices = gdb.parse_and_eval(NUM_DEVICES_FCT)
-            if num_devices == 0:
-                print("""intelgt: no GPU device is available for debug.""")
-                return
-
         if host_inf not in self.inf_dict or self.inf_dict[host_inf] is None:
             if not self.is_nonstop:
                 gdb.execute("set schedule-multiple off")
@@ -608,22 +578,29 @@ intelgt: the igfxdcd module (i.e. the debug driver) is not loaded.""")
             # Attach gdbserver to gt inferior.
             # This represents the first device.
             try:
-                self.handle_attach_gdbserver_gt(connection, host_inf)
+                if not self.handle_attach_gdbserver_gt(host_inf):
+                    return
+                # Get the most recent gt inferior.
+                gt_inf = gdb.inferiors()[-1]
+                # For the --attach scenario we use gt_inf; for the
+                # --multi scenario we use gt_inf's connection num.
+                self.inf_dict[host_inf] = (gt_inf, gt_inf.connection_num)
                 # Switch to the host inferior.
                 gdb.execute(f"inferior {host_inf.num}", False, True)
-                if not self.is_nonstop:
-                    gdb.execute("set schedule-multiple on")
             # Fix ctrl-c while attaching to gt inferior.
-            except (KeyboardInterrupt, gdb.error):
+            except KeyboardInterrupt:
                 self.handle_error(host_inf)
+                IntelgtErrorReport.exit_intelgt_session("Received ctrl-c interrupt.")
+            except gdb.error as ex:
+                # If auto attach fails due to uninitialized context, it
+                # should be retried by setting up the hook bp.
+                if "attempting to attach too early" in str(ex):
+                    self.handle_error(host_inf)
+                    self.setup_hook_bp()
+                else:
+                    self.handle_error(host_inf, details=str(ex))
+                    IntelgtErrorReport.exit_intelgt_session(exception=ex)
 
-            gt_inf = gdb.inferiors()[-1]
-            # For the --attach scenario we use gt_inf; for the
-            # --multi scenario we use gt_inf's connection num.
-            self.inf_dict[host_inf] = (gt_inf, gt_inf.connection_num)
-
-            # Switch to the host inferior.
-            gdb.execute(f"inferior {host_inf.num}", False, True)
             if not self.is_nonstop:
                 gdb.execute("set schedule-multiple on")
 
@@ -641,4 +618,7 @@ intelgt: the igfxdcd module (i.e. the debug driver) is not loaded.""")
 
         return was_cli_suppressed
 
-INTELGT_AUTO_ATTACH = IntelgtAutoAttach()
+AUTO_ATTACH_DISABLED = IntelgtAutoAttach.get_env_variable(
+        "INTELGT_AUTO_ATTACH_DISABLE", "0")
+if AUTO_ATTACH_DISABLED != "1":
+    INTELGT_AUTO_ATTACH = IntelgtAutoAttach()
