@@ -1,6 +1,6 @@
 /* Python interface to finish breakpoints
 
-   Copyright (C) 2011-2022 Free Software Foundation, Inc.
+   Copyright (C) 2011-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -90,9 +90,9 @@ bpfinishpy_dealloc (PyObject *self)
   Py_TYPE (self)->tp_free (self);
 }
 
-/* Triggered when gdbpy_should_stop is about to execute the `stop' callback
-   of the gdb.FinishBreakpoint object BP_OBJ.  Will compute and cache the
-   `return_value', if possible.  */
+/* Triggered when gdbpy_breakpoint_cond_says_stop is about to execute the `stop'
+   callback of the gdb.FinishBreakpoint object BP_OBJ.  Will compute and cache
+   the `return_value', if possible.  */
 
 void
 bpfinishpy_pre_stop_hook (struct gdbpy_breakpoint_object *bp_obj)
@@ -134,8 +134,8 @@ bpfinishpy_pre_stop_hook (struct gdbpy_breakpoint_object *bp_obj)
     }
 }
 
-/* Triggered when gdbpy_should_stop has triggered the `stop' callback
-   of the gdb.FinishBreakpoint object BP_OBJ.  */
+/* Triggered when gdbpy_breakpoint_cond_says_stop has triggered the `stop'
+   callback of the gdb.FinishBreakpoint object BP_OBJ.  */
 
 void
 bpfinishpy_post_stop_hook (struct gdbpy_breakpoint_object *bp_obj)
@@ -145,7 +145,7 @@ bpfinishpy_post_stop_hook (struct gdbpy_breakpoint_object *bp_obj)
     {
       /* Can't delete it here, but it will be removed at the next stop.  */
       disable_breakpoint (bp_obj->bp);
-      gdb_assert (bp_obj->bp->disposition == disp_del);
+      bp_obj->bp->disposition = disp_del_at_next_stop;
     }
   catch (const gdb_exception &except)
     {
@@ -164,8 +164,8 @@ bpfinishpy_init (PyObject *self, PyObject *args, PyObject *kwargs)
       (struct finish_breakpoint_object *) self;
   PyObject *frame_obj = NULL;
   int thread;
-  struct frame_info *frame = NULL; /* init for gcc -Wall */
-  struct frame_info *prev_frame = NULL;
+  frame_info_ptr frame = NULL; /* init for gcc -Wall */
+  frame_info_ptr prev_frame = NULL;
   struct frame_id frame_id;
   PyObject *internal = NULL;
   int internal_bp = 0;
@@ -205,12 +205,7 @@ bpfinishpy_init (PyObject *self, PyObject *args, PyObject *kwargs)
 				 "be set on a dummy frame."));
 	    }
 	  else
-	    {
-	      frame_id = get_frame_id (prev_frame);
-	      if (frame_id_eq (frame_id, null_frame_id))
-		PyErr_SetString (PyExc_ValueError,
-				 _("Invalid ID for the `frame' object."));
-	    }
+	    frame_id = get_frame_id (prev_frame);
 	}
     }
   catch (const gdb_exception &except)
@@ -254,7 +249,7 @@ bpfinishpy_init (PyObject *self, PyObject *args, PyObject *kwargs)
 	  if (function != nullptr)
 	    {
 	      struct type *ret_type =
-		check_typedef (TYPE_TARGET_TYPE (function->type ()));
+		check_typedef (function->type ()->target_type ());
 
 	      /* Remember only non-void return types.  */
 	      if (ret_type->code () != TYPE_CODE_VOID)
@@ -296,16 +291,16 @@ bpfinishpy_init (PyObject *self, PyObject *args, PyObject *kwargs)
   try
     {
       /* Set a breakpoint on the return address.  */
-      event_location_up location
-	= new_address_location (get_frame_pc (prev_frame), NULL, 0);
+      location_spec_up locspec
+	= new_address_location_spec (get_frame_pc (prev_frame), NULL, 0);
       create_breakpoint (gdbpy_enter::get_gdbarch (),
-			 location.get (), nullptr, thread, -1, nullptr, false,
+			 locspec.get (), nullptr, thread, -1, nullptr, false,
 			 0,
 			 1 /*temp_flag*/,
 			 bp_breakpoint,
 			 0,
 			 AUTO_BOOLEAN_TRUE,
-			 &bkpt_breakpoint_ops,
+			 &code_breakpoint_ops,
 			 0, 1, internal_bp, 0);
     }
   catch (const gdb_exception &except)
@@ -408,6 +403,9 @@ bpfinishpy_handle_exit (struct inferior *inf)
 int
 gdbpy_initialize_finishbreakpoints (void)
 {
+  if (!gdbpy_breakpoint_init_breakpoint_type ())
+    return -1;
+
   if (PyType_Ready (&finish_breakpoint_object_type) < 0)
     return -1;
 
