@@ -1,6 +1,6 @@
 /* Python interface to btrace instruction history.
 
-   Copyright 2016-2022 Free Software Foundation, Inc.
+   Copyright 2016-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -150,7 +150,12 @@ btpy_item_new (thread_info *tinfo, Py_ssize_t number)
   btrace_insn_iterator iter;
   int err_code;
 
-  btrace_find_insn_by_number (&iter, &tinfo->btrace, number);
+  if (btrace_find_insn_by_number (&iter, &tinfo->btrace, number) == 0)
+    {
+      PyErr_Format (gdbpy_gdb_error, _("No such instruction."));
+      return nullptr;
+    }
+
   err_code = btrace_insn_get_error (&iter);
 
   if (err_code != 0)
@@ -179,8 +184,8 @@ btpy_item_new (thread_info *tinfo, Py_ssize_t number)
 /* Create a new gdb.BtraceList object.  */
 
 static PyObject *
-btpy_list_new (thread_info *thread, Py_ssize_t first, Py_ssize_t last, Py_ssize_t step,
-	       PyTypeObject *element_type)
+btpy_list_new (thread_info *thread, Py_ssize_t first, Py_ssize_t last,
+	       Py_ssize_t step, PyTypeObject *element_type)
 {
   btpy_list_object * const obj = PyObject_New (btpy_list_object,
 					       &btpy_list_type);
@@ -320,7 +325,6 @@ recpy_bt_insn_decoded (PyObject *self, void *closure)
       return NULL;
     }
 
-
   return PyBytes_FromString (strfile.string ().c_str ());
 }
 
@@ -481,9 +485,9 @@ btpy_list_slice (PyObject *self, PyObject *value)
   const Py_ssize_t length = btpy_list_length (self);
   Py_ssize_t start, stop, step, slicelength;
 
-  if (PyInt_Check (value))
+  if (PyLong_Check (value))
     {
-      Py_ssize_t index = PyInt_AsSsize_t (value);
+      Py_ssize_t index = PyLong_AsSsize_t (value);
 
       /* Emulate Python behavior for negative indices.  */
       if (index < 0)
@@ -617,7 +621,7 @@ btpy_list_richcompare (PyObject *self, PyObject *other, int op)
 PyObject *
 recpy_bt_method (PyObject *self, void *closure)
 {
-  return PyString_FromString ("btrace");
+  return PyUnicode_FromString ("btrace");
 }
 
 /* Implementation of
@@ -638,7 +642,7 @@ recpy_bt_format (PyObject *self, void *closure)
   if (config == NULL)
     Py_RETURN_NONE;
 
-  return PyString_FromString (btrace_format_short_string (config->format));
+  return PyUnicode_FromString (btrace_format_short_string (config->format));
 }
 
 /* Implementation of
@@ -789,19 +793,19 @@ recpy_call_listener (const uint64_t *payload, const uint64_t *ip,
   if (PyErr_Occurred ())
     {
       gdbpy_print_stack ();
-      gdb_Py_DECREF (py_ip);
-      gdb_Py_DECREF (py_payload);
-      gdb_Py_DECREF (py_result);
+      Py_DECREF (py_ip);
+      Py_DECREF (py_payload);
+      Py_DECREF (py_result);
       PyGILState_Release (gstate);
       error (_("Error while executing Python code."));
     }
 
-  gdb_Py_DECREF (py_ip);
-  gdb_Py_DECREF (py_payload);
+  Py_DECREF (py_ip);
+  Py_DECREF (py_payload);
 
   if (py_result == Py_None)
     {
-      gdb_Py_DECREF (py_result);
+      Py_DECREF (py_result);
       PyGILState_Release (gstate);
       return nullptr;
     }
@@ -811,13 +815,13 @@ recpy_call_listener (const uint64_t *payload, const uint64_t *ip,
   if (PyErr_Occurred ())
     {
       gdbpy_print_stack ();
-      gdb_Py_DECREF (py_result);
+      Py_DECREF (py_result);
       PyGILState_Release (gstate);
       error (_("Error while executing Python code."));
     }
 
   if (py_result != nullptr)
-    gdb_Py_DECREF (py_result);
+    Py_DECREF (py_result);
   PyGILState_Release (gstate);
 
   return resultstring;
@@ -840,7 +844,7 @@ get_ptwrite_listener ()
   PyObject *default_ptw_listener = PyObject_CallMethod (module,	"get_listener",
 							NULL);
 
-  gdb_Py_DECREF (module);
+  Py_DECREF (module);
 
   if (PyErr_Occurred ())
     gdbpy_print_stack ();
