@@ -1,6 +1,6 @@
 /* Process record and replay target for GDB, the GNU debugger.
 
-   Copyright (C) 2008-2022 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -55,7 +55,7 @@ struct cmd_list_element *info_record_cmdlist = NULL;
 
 #define DEBUG(msg, args...)						\
   if (record_debug)							\
-    fprintf_unfiltered (gdb_stdlog, "record: " msg "\n", ##args)
+    gdb_printf (gdb_stdlog, "record: " msg "\n", ##args)
 
 /* See record.h.  */
 
@@ -257,8 +257,8 @@ static void
 show_record_debug (struct ui_file *file, int from_tty,
 		   struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Debugging of process record target is %s.\n"),
-		    value);
+  gdb_printf (file, _("Debugging of process record target is %s.\n"),
+	      value);
 }
 
 /* Alias for "target record".  */
@@ -279,14 +279,14 @@ cmd_record_delete (const char *args, int from_tty)
 
   if (!target_record_is_replaying (inferior_ptid))
     {
-      printf_filtered (_("Already at end of record list.\n"));
+      gdb_printf (_("Already at end of record list.\n"));
       return;
     }
 
   if (!target_supports_delete_record ())
     {
-      printf_filtered (_("The current record target does not support "
-			 "this operation.\n"));
+      gdb_printf (_("The current record target does not support "
+		    "this operation.\n"));
       return;
     }
 
@@ -308,8 +308,8 @@ cmd_record_stop (const char *args, int from_tty)
   record_stop (t);
   record_unpush (t);
 
-  printf_filtered (_("Process record is stopped and all execution "
-		     "logs are deleted.\n"));
+  gdb_printf (_("Process record is stopped and all execution "
+		"logs are deleted.\n"));
 
   gdb::observers::record_changed.notify (current_inferior (), 0, NULL, NULL);
 }
@@ -325,11 +325,11 @@ info_record_command (const char *args, int from_tty)
   t = find_record_target ();
   if (t == NULL)
     {
-      printf_filtered (_("No recording is currently active.\n"));
+      gdb_printf (_("No recording is currently active.\n"));
       return;
     }
 
-  printf_filtered (_("Active record target: %s\n"), t->shortname ());
+  gdb_printf (_("Active record target: %s\n"), t->shortname ());
   t->info_record ();
 }
 
@@ -494,6 +494,9 @@ get_insn_history_modifiers (const char **arg)
 	    case 'r':
 	      modifiers |= DISASSEMBLY_RAW_INSN;
 	      break;
+	    case 'b':
+	      modifiers |= DISASSEMBLY_RAW_BYTES;
+	      break;
 	    case 'f':
 	      modifiers |= DISASSEMBLY_OMIT_FNAME;
 	      break;
@@ -758,67 +761,6 @@ set_record_call_history_size (const char *args, int from_tty,
 			 &record_call_history_size);
 }
 
-void
-mi_cmd_record_function_call_history_length (const char *command,
-					    char **argv,
-					    int argc)
-{
-  if (argc != 0)
-    error (_("-function-call-history-length: Invalid number of arguments."));
-
-  target_call_history_length ();
-}
-
-void
-mi_cmd_record_function_call_history (const char *command, char **argv,
-				     int argc)
-{
-  record_print_flags flags = 0;
-  unsigned int low, high;
-  char *charend;
-
-  if ((argc < 0) || (argc > 3))
-    error (_("-function-call-history: Invalid number of arguments."));
-
-  require_record_target ();
-
-  if ((argc == 1) || (argc == 3))
-    {
-      const char *targv = argv[0];
-      flags = get_call_history_modifiers (&targv);
-    }
-
-  if ((argc == 0) || (argc == 1))
-    {
-      const int size = command_size_to_target_size (record_call_history_size);
-      target_call_history (size, flags);
-      return;
-    }
-
-  if (argc == 3)
-    {
-      low = strtoul (argv[1], &charend, 10);
-      if (*charend != '\0')
-	error (_("Invalid syntax of begin func id '%s'"), argv[1]);
-
-      high = strtoul (argv[2], &charend, 10);
-      if (*charend != '\0')
-	error (_("Invalid syntax of end func id '%s'"), argv[2]);
-    }
-  else
-    {
-      low = strtoul (argv[0], &charend, 10);
-      if (*charend != '\0')
-	error (_("Invalid syntax of begin func id '%s'"), argv[0]);
-
-      high = strtoul (argv[1], &charend, 10);
-      if (*charend != '\0')
-	error (_("Invalid syntax of end func id '%s'"), argv[1]);
-    }
-
-  target_call_history_range (low, high, flags);
-}
-
 void _initialize_record ();
 void
 _initialize_record ()
