@@ -1,6 +1,6 @@
 /* Process record and replay target for GDB, the GNU debugger.
 
-   Copyright (C) 2008-2022 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -117,15 +117,4 @@ extern void record_start (const char *method, const char *format,
 /* Stop recording.  Throw on failure.  */
 extern void record_stop (int from_tty);
 
-/* The MI version of the command to get the length of the function call history
-   for record targets.  */
-extern void mi_cmd_record_function_call_history_length (char *command,
-							char **argv,
-							int argc);
-
-/* The MI version of the command to get the function call history for record
-   targets.  */
-extern void mi_cmd_record_function_call_history (char *command, char **argv,
-						 int argc);
-
 #endif /* RECORD_H */
