@@ -1,5 +1,5 @@
 /* Target operations for the remote server for GDB.
-   Copyright (C) 2002-2022 Free Software Foundation, Inc.
+   Copyright (C) 2002-2023 Free Software Foundation, Inc.
 
    Contributed by MontaVista Software.
 
@@ -29,130 +29,68 @@
 
 process_stratum_target *the_target;
 
-int
+/* See target.h.  */
+
+bool
 set_desired_thread ()
 {
   client_state &cs = get_client_state ();
-  if (cs.general_thread.is_pid ())
+  thread_info *found = find_thread_ptid (cs.general_thread);
+
+  if (found == nullptr)
     {
       process_info *proc = find_process_pid (cs.general_thread.pid ());
-      switch_to_process (proc);
-      return (proc != nullptr);
-    }
-  else
-    {
-      thread_info *found = find_thread_ptid (cs.general_thread);
-      switch_to_thread (found);
-      return (current_thread != nullptr);
-    }
-}
-
-/* The thread that was current before prepare_to_access_memory was
-   called.  done_accessing_memory uses this to restore the previous
-   selected thread.  */
-static ptid_t prev_general_thread;
-
-/* See target.h.  */
-
-int
-prepare_to_access_memory (void)
-{
-  client_state &cs = get_client_state ();
-
-  /* The first thread found.  */
-  struct thread_info *first = NULL;
-  /* The first stopped thread found.  */
-  struct thread_info *stopped = NULL;
-  /* The current general thread, if found.  */
-  struct thread_info *current = NULL;
-
-  /* Save the general thread value, since prepare_to_access_memory could change
-     it.  */
-  prev_general_thread = cs.general_thread;
-
-  int res = the_target->prepare_to_access_memory ();
-  if (res != 0)
-    return res;
-
-  process_info *process = find_process_pid (prev_general_thread.pid ());
-  if (process == nullptr)
-    {
-      /* Undo the effects of target->prepare_to_access_memory() and
-	 return error.  */
-      done_accessing_memory ();
-      return 1;
-    }
-
-  for_each_thread (prev_general_thread.pid (), [&] (thread_info *thread)
-    {
-      if (mythread_alive (thread->id))
+      if (proc == nullptr)
 	{
-	  if (stopped == NULL && the_target->supports_thread_stopped ()
-	      && target_thread_stopped (thread))
-	    stopped = thread;
-
-	  if (first == NULL)
-	    first = thread;
-
-	  if (current == NULL && prev_general_thread == thread->id)
-	    {
-	      current = thread;
-	      /* If the current thread is also stopped, prefer it over
-		 other stopped threads.  */
-	      if (target_thread_stopped (current))
-		stopped = current;
-	    }
+	  threads_debug_printf
+	    ("did not find thread nor process for general_thread %s",
+	     cs.general_thread.to_string ().c_str ());
 	}
-    });
-
-  /* The thread we end up choosing.  */
-  struct thread_info *thread = nullptr;
-
-  /* Prefer a stopped thread.  If none is found, try the current
-     thread.  Otherwise, take the first thread in the process.  If
-     none is found, try the process.  Some architectures do not require
-     a particular thread to access the memory.  */
-  if (stopped != NULL)
-    thread = stopped;
-  else if (current != NULL)
-    thread = current;
-  else if (first != NULL)
-    thread = first;
-
-  if (thread != nullptr)
-    {
-      switch_to_thread (thread);
-      cs.general_thread = ptid_of (thread);
+      else
+	{
+	  threads_debug_printf
+	    ("did not find thread for general_thread %s, but found process",
+	     cs.general_thread.to_string ().c_str ());
+	}
+      switch_to_process (proc);
     }
   else
-    {
-      switch_to_process (process);
-      cs.general_thread = ptid_t {process->pid};
-    }
+    switch_to_thread (found);
 
-  return 0;
+  return (current_thread != NULL);
 }
 
 /* See target.h.  */
 
-void
-done_accessing_memory (void)
+bool
+set_desired_process ()
 {
   client_state &cs = get_client_state ();
 
-  the_target->done_accessing_memory ();
+  process_info *proc = find_process_pid (cs.general_thread.pid ());
+  if (proc == nullptr)
+    {
+      threads_debug_printf
+	("did not find process for general_thread %s",
+	 cs.general_thread.to_string ().c_str ());
+    }
+  switch_to_process (proc);
 
-  /* Restore the previous selected thread.  */
-  cs.general_thread = prev_general_thread;
-  set_desired_thread ();
+  return proc != nullptr;
 }
 
 int
 read_inferior_memory (CORE_ADDR memaddr, unsigned char *myaddr, int len,
 		      unsigned int addr_space)
 {
-  int res;
-  res = the_target->read_memory (memaddr, myaddr, len, addr_space);
+  /* At the time of writing, GDB only sends write packets with LEN==0,
+     not read packets (see comment in target_write_memory), but it
+     doesn't hurt to prevent problems if it ever does, or we're
+     connected to some client other than GDB that does.  */
+  if (len == 0)
+    return 0;
+
+  int res = the_target->read_memory (memaddr, myaddr, len, addr_space);
   check_mem_read (memaddr, myaddr, len);
   return res;
 }
@@ -180,6 +118,13 @@ int
 target_write_memory (CORE_ADDR memaddr, const unsigned char *myaddr,
 		     ssize_t len, unsigned int addr_space)
 {
+  /* GDB may send X packets with LEN==0, for probing packet support.
+     If we let such a request go through, then buffer.data() below may
+     return NULL, which may confuse target implementations.  Handle it
+     here to avoid lower levels having to care about this case.  */
+  if (len == 0)
+    return 0;
+
   /* Make a copy of the data because check_mem_write may need to
      update it.  */
   gdb::byte_vector buffer (myaddr, myaddr + len);
@@ -388,18 +333,6 @@ process_stratum_target::post_create_inferior ()
   /* Nop.  */
 }
 
-int
-process_stratum_target::prepare_to_access_memory ()
-{
-  return 0;
-}
-
-void
-process_stratum_target::done_accessing_memory ()
-{
-  /* Nop.  */
-}
-
 void
 process_stratum_target::look_up_symbols ()
 {
