@@ -1,6 +1,6 @@
 /* Multi-process/thread control for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2022 Free Software Foundation, Inc.
+   Copyright (C) 1986-2023 Free Software Foundation, Inc.
 
    Contributed by Lynx Real-Time Systems, Inc.  Los Gatos, CA.
 
@@ -50,6 +50,7 @@
 #include "gdbsupport/gdb_optional.h"
 #include "inline-frame.h"
 #include "stack.h"
+#include <array>
 
 /* See gdbthread.h.  */
 
@@ -61,7 +62,7 @@ static void
 show_debug_threads (struct ui_file *file, int from_tty,
 		    struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Thread debugging is \"%s\".\n"), value);
+  gdb_printf (file, _("Thread debugging is \"%s\".\n"), value);
 }
 
 /* Definition of struct thread_info exported to gdbthread.h.  */
@@ -101,7 +102,7 @@ thread_get_current_block (thread_info *tp)
   scoped_restore_current_thread restore_thread;
   switch_to_thread (tp);
 
-  frame_info * const frame = get_selected_frame (nullptr);
+  frame_info_ptr frame = get_selected_frame (nullptr);
   if (frame == nullptr)
     return nullptr;
 
@@ -138,7 +139,7 @@ thread_info::has_simd_lanes ()
       if (blk == nullptr)
 	return false;
 
-      return (BLOCK_SIMD_WIDTH (blk) > 0);
+      return (blk->simd_width () > 0);
     }
   catch (...)
    {
@@ -167,8 +168,8 @@ thread_info::active_simd_lanes_mask ()
       /* If the compiler indicated SIMD for the current block, we
 	 currently assume that all lanes are active.  */
       const block * const blk = thread_get_current_block (this);
-      if (blk != nullptr && BLOCK_SIMD_WIDTH (blk) > 0)
-	return ~(~0u << BLOCK_SIMD_WIDTH (blk));
+      if (blk != nullptr && blk->simd_width () > 0)
+	return ~(~0u << blk->simd_width ());
     }
 
   /* Default: only one lane is active.  */
@@ -249,7 +250,7 @@ thread_info::get_simd_width ()
 {
   const block *const blk = thread_get_current_block (this);
   if (blk != nullptr)
-    return BLOCK_SIMD_WIDTH (blk);
+    return blk->simd_width ();
 
   return 1;
 }
@@ -502,7 +503,7 @@ add_thread_with_info (process_stratum_target *targ, ptid_t ptid,
   result->priv.reset (priv);
 
   if (print_thread_events)
-    printf_unfiltered (_("[New %s]\n"), target_pid_to_str (ptid).c_str ());
+    gdb_printf (_("[New %s]\n"), target_pid_to_str (ptid).c_str ());
 
   annotate_new_thread ();
   return result;
@@ -1059,6 +1060,10 @@ set_running_thread (struct thread_info *tp, bool running)
     started = true;
   tp->state = running ? THREAD_RUNNING : THREAD_STOPPED;
 
+  threads_debug_printf ("thread: %s, running? %d%s",
+			tp->ptid.to_string ().c_str (), running,
+			(started ? " (started)" : ""));
+
   if (!running)
     {
       /* If the thread is now marked stopped, remove it from
@@ -1093,18 +1098,6 @@ set_running (process_stratum_target *targ, ptid_t ptid, bool running)
     if (set_running_thread (tp, running))
       any_started = true;
 
-  /* Pending attach targets are resumed even without threads present.
-     We check them here separately and set any_started in case we find a
-     pending attach target that we're going to resume.  */
-  for (process_stratum_target *target : all_process_targets ())
-    {
-      if (targ != nullptr && targ != target)
-	continue;
-      if (!target->pending_attach)
-	continue;
-      any_started = true;
-    }
-
   if (any_started)
     gdb::observers::target_resumed.notify (ptid);
 }
@@ -1387,17 +1380,28 @@ print_thread_row (struct ui_out *uiout, thread_info *tp,
 			     uiout->is_mi_like_p (),
 			     LOCATION, 0);
 
-	  /* SIMD specific fields for MI.  */
-	  if (uiout->is_mi_like_p () && tp->has_simd_lanes ())
+	  /* Fields for MI which are only for stopped available threads.  */
+	  if (uiout->is_mi_like_p ())
 	    {
-	      unsigned int mask = tp->active_simd_lanes_mask ();
-	      uiout->field_fmt ("execution-mask", "0x%x", mask);
-	      unsigned int width = tp->get_simd_width ();
-	      uiout->field_fmt ("simd-width", "%u", width);
-	      bpstat *bp = tp->control.stop_bpstat;
-	      if (bp != nullptr)
-		uiout->field_fmt ("hit-lanes-mask", "0x%x",
-				  bp->simd_lane_mask);
+	      if (tp->has_simd_lanes ())
+		{
+		  unsigned int mask = tp->active_simd_lanes_mask ();
+		  uiout->field_fmt ("execution-mask", "0x%x", mask);
+		  unsigned int width = tp->get_simd_width ();
+		  uiout->field_fmt ("simd-width", "%u", width);
+		  bpstat *bp = tp->control.stop_bpstat;
+		  unsigned int hit_lane_mask;
+		  if (bp != nullptr && bp->find_hit_lane_mask (hit_lane_mask))
+		    uiout->field_fmt ("hit-lanes-mask", "0x%x",
+				      hit_lane_mask);
+		}
+	      if (gdbarch_thread_workgroup_p (tp->inf->gdbarch))
+		{
+		  std::array<uint32_t, 3> group
+		    = gdbarch_thread_workgroup (tp->inf->gdbarch, tp);
+		  uiout->field_fmt ("thread-workgroup", "%u,%u,%u",
+				    group[0], group[1], group[2]);
+		}
 	    }
 	}
     }
@@ -1675,6 +1679,9 @@ info_threads_command_completer (struct cmd_list_element *ignore,
 void
 switch_to_thread_no_regs (struct thread_info *thread)
 {
+  gdb_assert (thread != nullptr);
+  threads_debug_printf ("thread = %s", thread->ptid.to_string ().c_str ());
+
   struct inferior *inf = thread->inf;
 
   set_current_program_space (inf->pspace);
@@ -1692,6 +1699,8 @@ switch_to_no_thread ()
   if (current_thread_ == nullptr)
     return;
 
+  threads_debug_printf ("thread = NONE");
+
   current_thread_ = nullptr;
   inferior_ptid = null_ptid;
   reinit_frame_cache ();
@@ -1930,8 +1939,8 @@ thread_try_catch_cmd (thread_info *thr, gdb::optional<int> ada_task,
       if (!flags.silent || cmd_result.length () > 0)
 	{
 	  if (!flags.quiet)
-	    printf_filtered ("%s", thr_header.c_str ());
-	  printf_filtered ("%s", cmd_result.c_str ());
+	    gdb_printf ("%s", thr_header.c_str ());
+	  gdb_printf ("%s", cmd_result.c_str ());
 	}
     }
   catch (const gdb_exception_error &ex)
@@ -1939,9 +1948,9 @@ thread_try_catch_cmd (thread_info *thr, gdb::optional<int> ada_task,
       if (!flags.silent)
 	{
 	  if (!flags.quiet)
-	    printf_filtered ("%s", thr_header.c_str ());
+	    gdb_printf ("%s", thr_header.c_str ());
 	  if (flags.cont)
-	    printf_filtered ("%s\n", ex.what ());
+	    gdb_printf ("%s\n", ex.what ());
 	  else
 	    throw;
 	}
@@ -2458,12 +2467,11 @@ thread_command (const char *tidstr, int from_tty)
 	  else if (tp->state == THREAD_EXITED)
 	    status_note = " (exited)";
 
-
-	  printf_filtered (_("[Current thread is %s (%s%s)%s]\n"),
-			   print_thread_id (tp, &lanes),
-			   target_pid_to_str (inferior_ptid).c_str (),
-			   lane_info.c_str (),
-			   status_note.c_str ());
+	  gdb_printf (_("[Current thread is %s (%s%s)%s]\n"),
+		      print_thread_id (tp, &lanes),
+		      target_pid_to_str (inferior_ptid).c_str (),
+		      lane_info.c_str (),
+		      status_note.c_str ());
 	}
       else
 	error (_("No stack."));
@@ -2541,37 +2549,37 @@ thread_find_command (const char *arg, int from_tty)
 
       if (tp->name () != nullptr && re_exec (tp->name ()))
 	{
-	  printf_filtered (_("Thread %s has name '%s'\n"),
-			   print_thread_id (tp), tp->name ());
+	  gdb_printf (_("Thread %s has name '%s'\n"),
+		      print_thread_id (tp), tp->name ());
 	  match++;
 	}
 
       tmp = target_thread_name (tp);
       if (tmp != NULL && re_exec (tmp))
 	{
-	  printf_filtered (_("Thread %s has target name '%s'\n"),
-			   print_thread_id (tp), tmp);
+	  gdb_printf (_("Thread %s has target name '%s'\n"),
+		      print_thread_id (tp), tmp);
 	  match++;
 	}
 
       std::string name = target_pid_to_str (tp->ptid);
       if (!name.empty () && re_exec (name.c_str ()))
 	{
-	  printf_filtered (_("Thread %s has target id '%s'\n"),
-			   print_thread_id (tp), name.c_str ());
+	  gdb_printf (_("Thread %s has target id '%s'\n"),
+		      print_thread_id (tp), name.c_str ());
 	  match++;
 	}
 
       tmp = target_extra_thread_info (tp);
       if (tmp != NULL && re_exec (tmp))
 	{
-	  printf_filtered (_("Thread %s has extra info '%s'\n"),
-			   print_thread_id (tp), tmp);
+	  gdb_printf (_("Thread %s has extra info '%s'\n"),
+		      print_thread_id (tp), tmp);
 	  match++;
 	}
     }
   if (!match)
-    printf_filtered (_("No threads match '%s'\n"), arg);
+    gdb_printf (_("No threads match '%s'\n"), arg);
 }
 
 /* Print notices when new threads are attached and detached.  */
@@ -2580,9 +2588,9 @@ static void
 show_print_thread_events (struct ui_file *file, int from_tty,
 			  struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file,
-		    _("Printing of thread events is %s.\n"),
-		    value);
+  gdb_printf (file,
+	      _("Printing of thread events is %s.\n"),
+	      value);
 }
 
 /* See gdbthread.h.  */
@@ -2808,6 +2816,22 @@ global_thread_id_make_value (struct gdbarch *gdbarch, struct internalvar *var,
   return thread_num_make_value_helper (gdbarch, 1);
 }
 
+/* Return a new value for the number of non-exited threads in the current
+   inferior.  If there are no threads in the current inferior return a
+   value of 0.  */
+
+static struct value *
+inferior_thread_count_make_value (struct gdbarch *gdbarch,
+				  struct internalvar *var, void *ignore)
+{
+  int int_val = 0;
+
+  if (inferior_ptid != null_ptid)
+    int_val = current_inferior ()->non_exited_threads ().size ();
+
+  return value_from_longest (builtin_type (gdbarch)->builtin_int, int_val);
+}
+
 /* Return a new value for the current SIMD lane of the selected thread.
    Return a value of -1 if no thread is selected, or no threads exist.  */
 
@@ -2835,6 +2859,75 @@ simd_width_make_value (gdbarch *gdbarch, internalvar *var, void *ignore)
 			     simd_width);
 }
 
+/* Return a new value with workgroup coordinates of the selected thread.
+   Return void if the workgroup is not defined or there is no thread
+   selected.  */
+static value *
+thread_workgroup_make_value (gdbarch *gdbarch, internalvar *var, void *ignore)
+{
+  const struct builtin_type *bt = builtin_type (gdbarch);
+  if (inferior_ptid == null_ptid || !gdbarch_thread_workgroup_p (gdbarch))
+    return allocate_value (bt->builtin_void);
+
+  thread_info *tp = inferior_thread ();
+  std::array<uint32_t, 3> workgroup_id
+    = gdbarch_thread_workgroup (gdbarch, tp);
+
+  type *result_type = init_vector_type (bt->builtin_unsigned_int, 3);
+  result_type->set_name ("_gdb_workitem");
+  value *val = value_from_contents (result_type,
+				    (gdb_byte *) (workgroup_id.data ()));
+
+  return val;
+}
+
+/* Return a new value with local coordinates of the selected lane within
+   its workgroup.  Return void if the workitem local ID is not defined or
+   there is no thread selected.  */
+
+static value *
+workitem_local_id_make_value (gdbarch *gdbarch, internalvar *var, void *ignore)
+{
+  const struct builtin_type *bt = builtin_type (gdbarch);
+  if (inferior_ptid == null_ptid || !gdbarch_workitem_local_id_p (gdbarch))
+    return allocate_value (bt->builtin_void);
+
+  thread_info *tp = inferior_thread ();
+  std::array<uint32_t, 3> local_id
+    = gdbarch_workitem_local_id (gdbarch, tp);
+
+  type *result_type = init_vector_type (bt->builtin_unsigned_int, 3);
+  result_type->set_name ("_gdb_workitem");
+  value *val = value_from_contents (result_type,
+				    (gdb_byte *) (local_id.data ()));
+
+  return val;
+}
+
+/* Return a new value with global coordinates of the selected lane.
+   Return void if the workitem global ID is not defined or there is no thread
+   selected.  */
+
+static value *
+workitem_global_id_make_value (gdbarch *gdbarch, internalvar *var,
+			       void *ignore)
+{
+  const struct builtin_type *bt = builtin_type (gdbarch);
+  if (inferior_ptid == null_ptid || !gdbarch_workitem_global_id_p (gdbarch))
+    return allocate_value (bt->builtin_void);
+
+  thread_info *tp = inferior_thread ();
+  std::array<uint32_t, 3> global_id
+    = gdbarch_workitem_global_id (gdbarch, tp);
+
+  type *result_type = init_vector_type (bt->builtin_unsigned_int, 3);
+  result_type->set_name ("_gdb_workitem");
+  value *val = value_from_contents (result_type,
+				    (gdb_byte *) (global_id.data ()));
+
+  return val;
+}
+
 /* Commands with a prefix of `thread'.  */
 struct cmd_list_element *thread_cmd_list = NULL;
 
@@ -2854,7 +2947,15 @@ static const struct internalvar_funcs gthread_funcs =
   NULL,
 };
 
-/* Implementation of `simd_lane' variable.  */
+/* Implementation of `_inferior_thread_count` convenience variable.  */
+
+static const struct internalvar_funcs inferior_thread_count_funcs =
+{
+  inferior_thread_count_make_value,
+  NULL,
+};
+
+/* Implementation of the `simd_lane' convenience variable.  */
 
 static const internalvar_funcs simd_lane_funcs =
 {
@@ -2870,6 +2971,30 @@ static const internalvar_funcs simd_width_funcs =
   nullptr,
 };
 
+/* Implementation of the `$_thread_workgroup' variable.  */
+
+static const internalvar_funcs thread_workgroup_funcs =
+{
+  thread_workgroup_make_value,
+  nullptr
+};
+
+/* Implementation of the `$_workitem_local_id' variable.  */
+
+static const internalvar_funcs workitem_local_id_funcs =
+{
+  workitem_local_id_make_value,
+  nullptr
+};
+
+/* Implementation of the `$_workitem_global_id' variable.  */
+
+static const internalvar_funcs workitem_global_id_funcs =
+{
+  workitem_global_id_make_value,
+  nullptr
+};
+
 void _initialize_thread ();
 void
 _initialize_thread ()
@@ -2898,7 +3023,10 @@ Options:\n\
   cmd_list_element *thread_cmd
     = add_prefix_cmd ("thread", class_run, thread_command, _("\
 Use this command to switch between threads.\n\
-The new thread ID must be currently known."),
+Usage: thread [ID][:LANE]\n\
+The new thread ID must be currently known.\n\
+For threads with SIMD lanes use additional LANE specifier to\n\
+switch for a specific lane of thread ID."),
 		      &thread_cmd_list, 1, &cmdlist);
 
   add_com_alias ("t", thread_cmd, class_run, 1);
@@ -2917,8 +3045,10 @@ Options:\n\
 
   static std::string thread_apply_help = gdb::option::build_help (_("\
 Apply a command to a list of threads.\n\
-Usage: thread apply ID... [OPTION]... COMMAND\n\
-ID is a space-separated list of IDs of threads to apply COMMAND on.\n"
+Usage: thread apply ID[:LANE]... [OPTION]... COMMAND\n\
+ID is a space-separated list of IDs of threads to apply COMMAND on.\n\
+For threads with SIMD lanes use additional :LANE specifier to\n\
+apply COMMAND to a lane range of thread ID.\n"
 THREAD_APPLY_OPTION_HELP),
 			       thread_apply_opts);
 
@@ -2998,6 +3128,14 @@ When on messages about thread creation and deletion are printed."),
 
   create_internalvar_type_lazy ("_thread", &thread_funcs, NULL);
   create_internalvar_type_lazy ("_gthread", &gthread_funcs, NULL);
+  create_internalvar_type_lazy ("_inferior_thread_count",
+				&inferior_thread_count_funcs, NULL);
   create_internalvar_type_lazy ("_simd_lane", &simd_lane_funcs, nullptr);
   create_internalvar_type_lazy ("_simd_width", &simd_width_funcs, nullptr);
+  create_internalvar_type_lazy ("_thread_workgroup",
+				&thread_workgroup_funcs, nullptr);
+  create_internalvar_type_lazy ("_workitem_local_id",
+				&workitem_local_id_funcs, nullptr);
+  create_internalvar_type_lazy ("_workitem_global_id",
+				&workitem_global_id_funcs, nullptr);
 }
