@@ -30,6 +30,7 @@
 #include <thread>
 #include <utility>
 #include <algorithm>
+#include <set>
 
 #ifndef USE_WIN32API
 #  include <signal.h>
@@ -379,7 +380,7 @@ ze_show_process (process_info *process)
       return;
     }
 
-  internal_error (__FILE__, __LINE__, _("unknown process state"));
+  internal_error (_("unknown process state"));
 }
 
 /* Hide PROCESS.  */
@@ -404,7 +405,7 @@ ze_hide_process (process_info *process)
       return;
     }
 
-  internal_error (__FILE__, __LINE__, _("unknown process state"));
+  internal_error (_("unknown process state"));
 }
 
 /* Attach to DEVICE and create a hidden process for it.
@@ -626,7 +627,7 @@ ze_set_resume_state (thread_info *tp, resume_kind rkind)
       return;
     }
 
-  internal_error (__FILE__, __LINE__, _("bad resume kind: %d."), rkind);
+  internal_error (_("bad resume kind: %d."), rkind);
 }
 
 /* Return TP's resume state.  */
@@ -940,12 +941,112 @@ ze_store_register (const ze_device_info &device,
 static void
 ze_discard_regcache (thread_info *tp)
 {
-  regcache *regcache = get_thread_regcache (tp, /* fetch = */ 0);
+  regcache *regcache = get_thread_regcache (tp, /* fetch = */ false);
   gdb_assert (regcache != nullptr);
 
   regcache->discard ();
 }
 
+/* Prepare for resuming TP.  Return true if TP should be actually
+   resumed.  */
+
+static bool
+ze_prepare_for_resuming (thread_info *tp)
+{
+  ze_thread_info *zetp = ze_thread (tp);
+  gdb_assert (zetp != nullptr);
+
+  /* We should not call this function if there is a priority
+     waitstatus.  */
+  gdb_assert (!ze_has_priority_waitstatus (tp));
+
+  /* When we get detached, we will remove the device but we will also mark
+     each thread exited.  We shouldn't try to resume them.  */
+  ze_device_info *device = ze_thread_device (tp);
+  gdb_assert (device != nullptr);
+
+  ze_thread_exec_state_t state = zetp->exec_state;
+  switch (state)
+    {
+    case ze_thread_state_stopped:
+      device->nresumed++;
+      if (device->nresumed > device->nthreads)
+	{
+	  device->nresumed = device->nthreads;
+	  dprintf ("capping device %lu's nresumed at %ld (all)",
+		   device->ordinal, device->nthreads);
+	}
+      return true;
+
+    case ze_thread_state_held:
+      gdb_assert_not_reached ("threads with 'held' state should "
+			      "have been turned into 'stopped'");
+
+    case ze_thread_state_unavailable:
+      device->nresumed++;
+      if (device->nresumed > device->nthreads)
+	{
+	  device->nresumed = device->nthreads;
+	  dprintf ("capping device %lu's nresumed at %ld (all)",
+		   device->ordinal, device->nthreads);
+	}
+
+      zetp->exec_state = ze_thread_state_running;
+
+      /* Ignore resuming unavailable threads.  */
+      return false;
+
+    case ze_thread_state_running:
+      /* Ignore resuming already running threads.  */
+      return false;
+
+    case ze_thread_state_unknown:
+      warning (_("thread %d.%ld has unknown execution "
+		 "state"), tp->id.pid (), tp->id.lwp ());
+      return false;
+    }
+
+  internal_error (_("bad execution state: %d."), state);
+}
+
+/* Prepare for stopping TP.  Return true if TP should be
+   actually stopped by sending an interrupt to the target.  */
+
+static bool
+ze_prepare_for_stopping (thread_info *tp)
+{
+  ze_thread_info *zetp = ze_thread (tp);
+  gdb_assert (zetp != nullptr);
+
+  /* When we get detached, we will remove the device but we will also mark
+     each thread exited.  We shouldn't try to stop them.  */
+  ze_device_info *device = ze_thread_device (tp);
+  gdb_assert (device != nullptr);
+
+  ze_thread_exec_state_t state = zetp->exec_state;
+  switch (state)
+    {
+    case ze_thread_state_stopped:
+      /* We silently ignore already stopped threads.  */
+      return false;
+
+    case ze_thread_state_held:
+      gdb_assert_not_reached ("threads with 'held' state should "
+			      "have been turned into 'stopped'");
+
+    case ze_thread_state_unavailable:
+    case ze_thread_state_running:
+      return true;
+
+    case ze_thread_state_unknown:
+      warning (_("thread %d.%ld has unknown execution "
+		 "state"), tp->id.pid (), tp->id.lwp ());
+      return false;
+    }
+
+  internal_error (_("bad execution state: %d."), state);
+}
+
 /* Resume THREAD on DEVICE.  */
 
 static void
@@ -1168,7 +1269,7 @@ ze_target::attach_to_device (uint32_t pid, ze_device_handle_t device)
 
   unsigned long ordinal = this->ordinal + 1;
   if (ordinal == 0)
-    internal_error (__FILE__, __LINE__, _("device ordinal overflow."));
+    internal_error (_("device ordinal overflow."));
 
   dinfo->ordinal = ordinal;
 
@@ -1470,7 +1571,14 @@ ze_target::fetch_events (ze_device_info &device)
 				 ze_thread_id_str (zetp->id).c_str ());
 
 			zetp->waitstatus.set_ignore ();
-			resume (tp, resume_continue);
+			ze_set_resume_state (tp, resume_continue);
+
+			bool should_resume = ze_prepare_for_resuming (tp);
+			gdb_assert (should_resume);
+			prepare_thread_resume (tp);
+			regcache_invalidate_thread (tp);
+			ze_resume (device, zetp->id);
+
 			return;
 		      }
 
@@ -1691,7 +1799,18 @@ ze_target::attach (unsigned long pid)
 	  /* GDB (and higher layers of gdbserver) expects threads stopped on
 	     attach in all-stop mode.  */
 	  if (!non_stop)
-	    resume (*device, resume_stop);
+	    {
+	      int device_pid = ze_device_pid (*device);
+	      for_each_thread (device_pid, [this] (thread_info *tp)
+		{
+		  ze_set_resume_state (tp, resume_stop);
+		  bool should_stop = ze_prepare_for_stopping (tp);
+		  gdb_assert (should_stop);
+		});
+
+	      ze_device_thread_t all = ze_thread_id_all ();
+	      ze_interrupt (*device, all);
+	    }
 
 	  nattached += 1;
 	  break;
@@ -1753,7 +1872,7 @@ ze_target::detach (process_info *proc)
 	      (void) ze_move_waitstatus (tp);
 	    });
 
-	  resume (*device, resume_continue);
+	  resume (*device);
 	}
       catch (const gdb_exception_error &except)
 	{
@@ -1788,214 +1907,33 @@ ze_target::join (int pid)
 }
 
 void
-ze_target::resume (ze_device_info &device, enum resume_kind rkind)
+ze_target::resume (ze_device_info &device)
 {
-  switch (rkind)
+  gdb_assert (device.process != nullptr);
+
+  bool has_thread_to_resume = false;
+  for_each_thread (ze_device_pid (device), [&] (thread_info *tp)
     {
-    case resume_continue:
-    case resume_step:
-      {
-	int pid = ze_device_pid (device);
-
-	/* We come here if we have nothing pending to report.  We can
-	   simply resume everything with a single resume request for
-	   all-stop and non-stop.  */
-	uint32_t nstopped = 0;
-	for_each_thread (pid, [this, rkind, &nstopped] (thread_info *tp)
-	  {
-	    ze_set_resume_state (tp, rkind);
-
-	    enum ze_thread_exec_state_t state = ze_exec_state (tp);
-	    switch (state)
-	      {
-	      case ze_thread_state_stopped:
-		/* We have reported the previous stop.  */
-		gdb_assert (!ze_has_waitstatus (tp));
-
-		nstopped += 1;
-		switch (rkind)
-		  {
-		  case resume_continue:
-		  case resume_step:
-		    prepare_thread_resume (tp, rkind);
-		    regcache_invalidate_thread (tp);
-		    return;
-
-		  default:
-		    gdb_assert_not_reached ("unexpected resume kind");
-		  }
-
-	      case ze_thread_state_held:
-		gdb_assert_not_reached ("threads with 'held' state should "
-					"have been turned into 'stopped'");
-
-	      case ze_thread_state_unavailable:
-		/* The thread is still running for all we know.  */
-		gdb_assert (!ze_has_waitstatus (tp));
-		return;
-
-	      case ze_thread_state_running:
-		/* The thread doesn't have anything to report.  */
-		gdb_assert (!ze_has_waitstatus (tp));
-		return;
-
-	      case ze_thread_state_unknown:
-		warning (_("thread %d.%ld has unknown execution state"),
-			 tp->id.pid (), tp->id.lwp ());
-		return;
-	      }
-
-	    internal_error (__FILE__, __LINE__,
-			    _("bad execution state: %d."), state);
-	  });
-
-	/* All threads are potentially running from our point of view.  */
-	device.nresumed = device.nthreads;
-	dprintf ("device %lu's nresumed=%ld%s",
-		 device.ordinal, device.nresumed,
-		 ((device.nresumed == device.nthreads) ? " (all)" : ""));
-
-	/* There is nothing to resume if nothing is stopped.  */
-	if (nstopped == 0)
-	  return;
-
-	ze_device_thread_t all = ze_thread_id_all ();
-	ze_resume (device, all);
-	return;
-      }
-
-    case resume_stop:
-      {
-	/* The stop case is comparably easy since the target will ignore
-	   already stopped threads.  */
-	int pid = ze_device_pid (device);
-	for_each_thread (pid, [this, rkind] (thread_info *tp)
-	  {
-	    ze_set_resume_state (tp, rkind);
-	  });
-
-	ze_device_thread_t all = ze_thread_id_all ();
-	ze_interrupt (device, all);
-	return;
-      }
-    }
-
-  internal_error (__FILE__, __LINE__, _("bad resume kind: %d."), rkind);
-}
-
-void
-ze_target::resume (thread_info *tp, enum resume_kind rkind)
-{
-  ze_thread_info *zetp = ze_thread (tp);
-  gdb_assert (zetp != nullptr);
-
-  /* When we get detached, we will remove the device but we will also mark
-     each thread exited.  We shouldn't try to resume them.  */
-  ze_device_info *device = ze_thread_device (tp);
-  gdb_assert (device != nullptr);
-
-  ze_thread_exec_state_t state = zetp->exec_state;
-  switch (state)
-    {
-    case ze_thread_state_stopped:
-      ze_set_resume_state (tp, rkind);
-
-      /* We silently ignore threads that still need to report an event.
-
-	 We still need to keep the resume state set, so we will
-	 actually report the event.  */
-      if (ze_has_priority_waitstatus (tp))
-	return;
-
-      switch (rkind)
+      ze_set_resume_state (tp, resume_continue);
+      if (ze_prepare_for_resuming (tp))
 	{
-	case resume_continue:
-	case resume_step:
-	  prepare_thread_resume (tp, rkind);
-	  if (device->nresumed < device->nthreads)
-	    {
-	      device->nresumed++;
-	      dprintf ("device %lu's nresumed=%ld%s", device->ordinal,
-		       device->nresumed,
-		       ((device->nresumed == device->nthreads)
-			? " (all)" : ""));
-	    }
-	  else
-	    {
-	      device->nresumed = device->nthreads;
-	      dprintf ("capping device %lu's nresumed at %ld (all)",
-		       device->ordinal, device->nthreads);
-	    }
+	  prepare_thread_resume (tp);
 	  regcache_invalidate_thread (tp);
-	  ze_resume (*device, zetp->id);
-	  return;
-
-	case resume_stop:
-	  /* We silently ignore already stopped threads.  */
-	  return;
+	  has_thread_to_resume = true;
 	}
+    });
 
-      internal_error (__FILE__, __LINE__, _("bad resume kind: %d."), rkind);
+  /* There is nothing to resume if nothing is stopped.  */
+  if (!has_thread_to_resume)
+    return;
 
-    case ze_thread_state_held:
-      gdb_assert_not_reached ("threads with 'held' state should "
-			      "have been turned into 'stopped'");
-
-    case ze_thread_state_unavailable:
-      dprintf ("resume request for unavailable thread %d.%ld (%s)",
-	       tp->id.pid (), tp->id.lwp (),
-	       ze_thread_id_str (zetp->id).c_str ());
-      if (device->nresumed < device->nthreads)
-	{
-	  device->nresumed++;
-	  dprintf ("device %lu's nresumed=%ld%s", device->ordinal,
-		   device->nresumed,
-		   ((device->nresumed == device->nthreads)
-		    ? " (all)" : ""));
-	}
-      else
-	{
-	  device->nresumed = device->nthreads;
-	  dprintf ("capping device %lu's nresumed at %ld (all)",
-		   device->ordinal, device->nthreads);
-	}
-
-      /* Fall-through.  */
-
-    case ze_thread_state_running:
-      ze_set_resume_state (tp, rkind);
-
-      switch (rkind)
-	{
-	case resume_continue:
-	case resume_step:
-	  /* Silently ignore already running or unavailable threads.  */
-	  return;
-
-	case resume_stop:
-	  ze_interrupt (*device, zetp->id);
-	  return;
-	}
-
-      internal_error (__FILE__, __LINE__, _("bad resume kind: %d."), rkind);
-
-    case ze_thread_state_unknown:
-      warning (_("thread %d.%ld has unknown execution "
-		 "state"), tp->id.pid (), tp->id.lwp ());
-      return;
-    }
-
-  internal_error (__FILE__, __LINE__, _("bad execution state: %d."), state);
+  ze_device_thread_t all = ze_thread_id_all ();
+  ze_resume (device, all);
 }
 
 size_t
 ze_target::mark_eventing_threads (ptid_t resume_ptid, resume_kind rkind)
 {
-  /* If resume_ptid is a wildcard PTID, convert it to a PID so that
-     'matches' will work.  */
-  if (resume_ptid.lwp () == -1 && resume_ptid.pid () > 0)
-    resume_ptid = ptid_t (resume_ptid.pid (), 0, 0);
-
   /* Note that even if we stopped all, unavailable threads may still
      report new events as we were not able to stop them.
 
@@ -2051,6 +1989,117 @@ ze_target::mark_eventing_threads (ptid_t resume_ptid, resume_kind rkind)
   return num_eventing;
 }
 
+/* Display a resume request for logging purposes.  */
+
+static void
+print_resume_info (const thread_resume &rinfo)
+{
+  ptid_t rptid = rinfo.thread;
+
+  switch (rinfo.kind)
+    {
+    case resume_continue:
+      dprintf ("received 'continue' resume request for (%d, %ld, %ld)",
+	       rptid.pid (), rptid.lwp (), rptid.tid ());
+      return;
+
+    case resume_step:
+      dprintf ("received 'step' resume request for (%d, %ld, %ld)"
+	       " in range [0x%" PRIx64 ", 0x%" PRIx64 ")",
+	       rptid.pid (), rptid.lwp (), rptid.tid (),
+	       rinfo.step_range_start, rinfo.step_range_end);
+      return;
+
+    case resume_stop:
+      dprintf ("received 'stop' resume request for (%d, %ld, %ld)",
+	       rptid.pid (), rptid.lwp (), rptid.tid ());
+      return;
+    }
+
+  internal_error (_("bad resume kind: %d."), rinfo.kind);
+}
+
+/* Normalize the resume requests for easier processing later on.  */
+
+static void
+normalize_resume_infos (thread_resume *resume_info, size_t n)
+{
+  for (size_t i = 0; i < n; ++i)
+    {
+      thread_resume &rinfo = resume_info[i];
+      ptid_t rptid = rinfo.thread;
+
+      /* Log the original requests.  */
+      print_resume_info (rinfo);
+
+      /* We convert ptids of the form (p, -1, 0) to (p, 0, 0) to make
+	 'ptid.matches' work.  This transformation is safe because we
+	 enumerate the threads starting at 1.  */
+      if ((rptid.lwp () == -1) && (rptid.pid () > 0))
+	rinfo.thread = ptid_t (rptid.pid (), 0, 0);
+
+      if (rinfo.sig != 0)
+	{
+	  /*  Clear out the signal.  Our target does not accept
+	      signals.  */
+	  warning (_("Ignoring signal on resume request for %d.%ld"),
+		   rinfo.thread.pid (), rinfo.thread.lwp ());
+	  rinfo.sig = 0;
+	}
+    }
+}
+
+/* Resuming threads of a device all at once with a single API call
+   is preferable to resuming threads individually.  Therefore, we
+   want to combine individual resume requests with wildcard resumes,
+   if possible.
+
+   For instance, if we receive "vCont;s:1;s:2;c", we would like to
+   make a single ze_resume call with the 'all.all.all.all' thread id
+   after preparing threads 1 and 2 for stepping and the others for
+   continuing.
+
+   We preprocess the resume requests to find for which devices we
+   shall combine the requests.  We attempt a merge in all-stop mode
+   when the requests contain continue/step requests only.  */
+
+static std::set<ze_device_info *>
+find_wildcard_devices (thread_resume *resume_info, size_t n,
+		       const std::list<ze_device_info *> &devices)
+{
+  std::set<ze_device_info *> wildcard_devices;
+
+  if (non_stop)
+    return wildcard_devices;
+
+  for (size_t i = 0; i < n; ++i)
+    {
+      if (resume_info[i].kind == resume_stop)
+	{
+	  wildcard_devices.clear ();
+	  break;
+	}
+
+      ptid_t rptid = resume_info[i].thread;
+      if (rptid == minus_one_ptid)
+	{
+	  for (ze_device_info *device : devices)
+	    wildcard_devices.insert (device);
+	  break;
+	}
+
+      if (rptid.is_pid ())
+	{
+	  process_info *proc = find_process_pid (rptid.pid ());
+	  ze_device_info *device = ze_process_device (proc);
+	  if (device != nullptr)
+	    wildcard_devices.insert (device);
+	}
+    }
+
+  return wildcard_devices;
+}
+
 void
 ze_target::resume (thread_resume *resume_info, size_t n)
 {
@@ -2076,6 +2125,8 @@ ze_target::resume (thread_resume *resume_info, size_t n)
 	ze_clear_resume_state (tp);
       });
 
+  normalize_resume_infos (resume_info, n);
+
   /* Check if there is a thread with a pending event for any of the
      resume requests.  In all-stop mode, we would omit actually
      resuming the target if there is such a thread.  In non-stop mode,
@@ -2096,28 +2147,92 @@ ze_target::resume (thread_resume *resume_info, size_t n)
   if ((num_eventing > 0) && !non_stop)
     return;
 
-  /* Let's hope that we're not getting any conflicting resume requests.
+  std::set<ze_device_info *> wildcard_devices
+    = find_wildcard_devices (resume_info, n, devices);
 
-     If higher layers cannot guarantee that, we'd need to add a check, but
-     this should really be the responsibility of higher layers, IMHO.  */
+  std::set<ze_device_info *> devices_to_resume;
+
+  /* Lambda for applying a resume info on a single thread.  */
+  auto apply_resume_info = ([&] (const thread_resume &rinfo,
+				 thread_info *tp)
+    {
+      if (ze_has_priority_waitstatus (tp))
+	return;
+
+      ze_set_resume_state (tp, rinfo.kind);
+      ze_device_info *device = ze_thread_device (tp);
+      ze_device_thread_t tid = ze_thread_id (tp);
+
+      switch (rinfo.kind)
+	{
+	case resume_stop:
+	  if (ze_prepare_for_stopping (tp))
+	    ze_interrupt (*device, tid);
+	  break;
+
+	case resume_step:
+	  {
+	    ze_thread_info *zetp = ze_thread (tp);
+	    gdb_assert (zetp != nullptr);
+
+	    regcache *regcache
+	      = get_thread_regcache (tp, /* fetch = */ false);
+	    CORE_ADDR pc = read_pc (regcache);
+
+	    /* For single-stepping, start == end.  Typically, both are 0.
+	       For range-stepping, the PC must be within the range.  */
+	    CORE_ADDR start = rinfo.step_range_start;
+	    CORE_ADDR end = rinfo.step_range_end;
+	    gdb_assert ((start == end) || ((pc >= start) && (pc < end)));
+
+	    zetp->step_range_start = start;
+	    zetp->step_range_end = end;
+	  }
+	  /* Fall through.  */
+
+	case resume_continue:
+	  if (ze_prepare_for_resuming (tp))
+	    {
+	      prepare_thread_resume (tp);
+	      regcache_invalidate_thread (tp);
+
+	      /* If the device can be resumed as a whole,
+		 omit resuming the thread individually.  */
+	      if (wildcard_devices.count (device) == 0)
+		ze_resume (*device, tid);
+	      else
+		devices_to_resume.insert (device);
+	    }
+	  break;
+	}
+    });
+
+  /* We may receive multiple requests that apply to a thread.  E.g.
+     "vCont;r0xff10,0xffa0:p1.9;c" could be sent to make thread 1.9 do
+     range-stepping from 0xff10 to 0xffa0, while continuing others.
+     According to the Remote Protocol Section E.2 (Packets),
+     "For each inferior thread, the leftmost action with a matching
+     thread-id is applied."  For this reason, we keep track of which
+     threads have been resumed individually so that we can skip them
+     when processing wildcard requests.
+
+     Alternatively, we could have the outer loop iterate over threads
+     and the inner loop iterate over resume infos to find the first
+     matching resume info for each thread.  There may, however, be a
+     large number of threads and a handful of resume infos that apply
+     to a few threads only.  For performance reasons, we prefer to
+     iterate over resume infos in the outer loop.  */
+  std::set<thread_info *> individually_resumed_threads;
   for (size_t i = 0; i < n; ++i)
     {
       const thread_resume &rinfo = resume_info[i];
-      if (rinfo.sig != 0)
-	warning (_("ignoring signal on resume request for %d.%ld"),
-		 rinfo.thread.pid (), rinfo.thread.lwp ());
-
+      gdb_assert (rinfo.sig == 0);
       ptid_t rptid = rinfo.thread;
       int rpid = rptid.pid ();
       if ((rptid == minus_one_ptid)
 	  || rptid.is_pid ()
 	  || (rptid.lwp () == -1))
 	{
-	  if ((rinfo.kind == resume_step)
-	      && (rinfo.step_range_start != rinfo.step_range_end))
-	    warning (_("ignoring range step request for %d.%ld"),
-		     rinfo.thread.pid (), rinfo.thread.lwp ());
-
 	  for (ze_device_info *device : devices)
 	    {
 	      gdb_assert (device != nullptr);
@@ -2126,62 +2241,28 @@ ze_target::resume (thread_resume *resume_info, size_t n)
 	      if ((rpid != -1) && (rpid != pid))
 		continue;
 
-	      resume_kind rkind = rinfo.kind;
-
-	      /* In non-stop mode, resume the threads individually,
-		 if there are threads with pending events.  To reduce
-		 the number of resume requests on the target, we may
-		 try to find a wildcard thread id that covers the
-		 range of the stopped threads, but we are not doing
-		 this at the moment.  */
-	      if (non_stop && (num_eventing > 0))
+	      for_each_thread (pid, [&] (thread_info *tp)
 		{
-		  for_each_thread (pid, [this, rkind] (thread_info *tp)
-		    {
-		      if (!ze_has_priority_waitstatus (tp))
-			resume (tp, rkind);
-		    });
-		}
-	      else
-		resume (*device, rkind);
+		  /* We trust that GDB will not send us wildcard resume
+		     requests with overlapping pids.  Hence, we track
+		     only individually-resumed threads.  */
+		  if (individually_resumed_threads.count (tp) == 0)
+		    apply_resume_info (rinfo, tp);
+		});
 	    }
 	}
       else
 	{
 	  thread_info *tp = find_thread_ptid (rptid);
-	  if (!ze_has_priority_waitstatus (tp))
-	    {
-	      if (rinfo.kind == resume_step)
-		{
-		  ze_thread_info *zetp = ze_thread (tp);
-		  gdb_assert (zetp != nullptr);
-
-		  regcache *regcache
-		    = get_thread_regcache (tp, /* fetch = */ false);
-		  CORE_ADDR pc = read_pc (regcache);
-
-		  /* For single-stepping, start == end.  Typically,
-		     both are 0.  For range-stepping, the PC must be
-		     within the range.  */
-		  CORE_ADDR start = rinfo.step_range_start;
-		  CORE_ADDR end = rinfo.step_range_end;
-		  gdb_assert ((start == end)
-			      || ((pc >= start) && (pc < end)));
-
-		  zetp->step_range_start = start;
-		  zetp->step_range_end = end;
-
-		  if (start != end)
-		    dprintf ("resuming thread %s for stepping in range "
-			     "[0x%" PRIx64 ", 0x%" PRIx64 ")",
-			     ze_thread_id_str (zetp->id).c_str (),
-			     start, end);
-		}
-
-	      resume (tp, rinfo.kind);
-	    }
+	  apply_resume_info (rinfo, tp);
+	  individually_resumed_threads.insert (tp);
 	}
     }
+
+  /* Finally, resume the whole devices.  */
+  ze_device_thread_t all = ze_thread_id_all ();
+  for (ze_device_info *device : devices_to_resume)
+    ze_resume (*device, all);
 }
 
 /* Look for a thread preferably with a priority stop
@@ -2377,7 +2458,15 @@ ze_target::wait (ptid_t ptid, target_waitstatus *status,
 		       zetp->step_range_start, zetp->step_range_end);
 
 	      (void) ze_move_waitstatus (thread);
-	      resume (thread, resume_step);
+	      gdb_assert (zetp->resume_state == ze_thread_resume_step);
+
+	      ze_device_info *device = ze_thread_device (thread);
+	      bool should_resume = ze_prepare_for_resuming (thread);
+	      gdb_assert (should_resume);
+	      prepare_thread_resume (thread);
+	      regcache_invalidate_thread (thread);
+	      ze_resume (*device, zetp->id);
+
 	      continue;
 	    }
 
@@ -2593,7 +2682,7 @@ ze_target::pause_all (bool freeze)
   if (freeze)
     {
       if (frozen == UINT32_MAX)
-	internal_error (__FILE__, __LINE__, _("freeze count overflow"));
+	internal_error (_("freeze count overflow"));
       frozen += 1;
     }
 
@@ -2695,8 +2784,7 @@ ze_target::pause_all (bool freeze)
 	    return true;
 	  }
 
-	internal_error (__FILE__, __LINE__, _("bad execution state: %d."),
-			state);
+	internal_error (_("bad execution state: %d."), state);
       });
   } while (thread != nullptr);
 
@@ -2716,7 +2804,7 @@ ze_target::unpause_all (bool unfreeze)
   if (unfreeze)
     {
       if (frozen == 0)
-	internal_error (__FILE__, __LINE__, _("freeze count underflow"));
+	internal_error (_("freeze count underflow"));
       frozen -= 1;
     }
 
@@ -2757,51 +2845,7 @@ ze_target::unpause_all (bool unfreeze)
 	}
     });
 
-  if (non_stop)
-    {
-      /* Let's resume threads by device and try to group them.  */
-      for (ze_device_info *device : devices)
-	{
-	  gdb_assert (device != nullptr);
-
-	  /* Ignore devices we're not modelling as processes.  */
-	  process_info *process = device->process;
-	  if (process == nullptr)
-	    continue;
-
-	  /* Check if we have any thread with an event to report.  */
-	  thread_info *thread
-	    = find_thread (pid_of (process), [] (thread_info *tp)
-		{
-		  return ze_has_priority_waitstatus (tp);
-		});
-
-	  /* If we don't, we can group all threads together.
-
-	     Otherwise, we send individual resume requests.  Let's not try
-	     to group them any further for now.  */
-	  if (thread == nullptr)
-	    {
-	      ze_device_thread_t all = ze_thread_id_all ();
-	      ze_resume (*device, all);
-	    }
-	  else
-	    {
-	      for_each_thread (pid_of (process), [device] (thread_info *tp)
-		{
-		  const ze_thread_info *zetp = ze_thread (tp);
-		  gdb_assert (zetp != nullptr);
-
-		  if (ze_has_priority_waitstatus (tp))
-		    return;
-
-		  ze_device_thread_t id = ze_thread_id (tp);
-		  ze_resume (*device, id);
-		});
-	    }
-	}
-    }
-  else
+  if (!non_stop)
     {
       /* Check whether we have events we have not reported, yet.
 
@@ -2818,19 +2862,18 @@ ze_target::unpause_all (bool unfreeze)
 	 We will report the event in the next wait ().  */
       if (thread != nullptr)
 	return;
+    }
 
-      /* There are no unreported events.  Let's resume everything.  */
-      ze_device_thread_t all = ze_thread_id_all ();
-      for (ze_device_info *device : devices)
-	{
-	  gdb_assert (device != nullptr);
+  /* Let's resume threads by device.  */
+  for (ze_device_info *device : devices)
+    {
+      gdb_assert (device != nullptr);
 
-	  /* Ignore devices we're not modelling as processes.  */
-	  if (device->process == nullptr)
-	    continue;
+      /* Ignore devices we're not modelling as processes.  */
+      if (device->process == nullptr)
+	continue;
 
-	  ze_resume (*device, all);
-	}
+      resume (*device);
     }
 }
 
